@@ -14,13 +14,11 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <errno.h>
 
-#include <glib/gprintf.h>
-
 #include "yad.h"
 
 static GtkWidget *color;
@@ -95,6 +93,14 @@ create_palette ()
   return GTK_TREE_MODEL (store);
 }
 
+static void
+set_color (gchar *clr)
+{
+  GdkRGBA c;
+  if (gdk_rgba_parse (&c, clr))
+    gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (color), &c);
+}
+
 static void
 palette_changed (GtkTreeSelection * sel, gpointer data)
 {
@@ -104,11 +110,8 @@ palette_changed (GtkTreeSelection * sel, gpointer data)
   if (gtk_tree_selection_get_selected (sel, &model, &iter))
     {
       gchar *clr;
-      GdkColor c;
-
       gtk_tree_model_get (model, &iter, 1, &clr, -1);
-      if (gdk_color_parse (clr, &c))
-        gtk_color_selection_set_current_color (GTK_COLOR_SELECTION (color), &c);
+      set_color (clr);
     }
 }
 
@@ -117,22 +120,13 @@ color_create_widget (GtkWidget * dlg)
 {
   GtkWidget *w;
 
-#if !GTK_CHECK_VERSION(3,0,0)
-  w = gtk_vbox_new (FALSE, 2);
-#else
   w = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-#endif
-
-  color = gtk_color_selection_new ();
+  color = gtk_color_chooser_widget_new ();
   gtk_widget_set_name (color, "yad-color-widget");
-  gtk_color_selection_set_has_palette (GTK_COLOR_SELECTION (color), options.color_data.gtk_palette);
-  if (options.color_data.init_color)
-    {
-      GdkColor c;
+  g_object_set (G_OBJECT (color), "show-editor", !options.color_data.gtk_palette, NULL);
 
-      if (gdk_color_parse (options.color_data.init_color, &c))
-        gtk_color_selection_set_current_color (GTK_COLOR_SELECTION (color), &c);
-    }
+  if (options.color_data.init_color)
+    set_color (options.color_data.init_color);
   gtk_box_pack_start (GTK_BOX (w), color, FALSE, FALSE, 2);
 
   if (options.color_data.use_palette)
@@ -158,7 +152,7 @@ color_create_widget (GtkWidget * dlg)
           gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), options.hscroll_policy, options.vscroll_policy);
           gtk_widget_set_size_request (sw, -1, 75);
           gtk_container_add (GTK_CONTAINER (exp), sw);
-
+          gtk_widget_set_vexpand (sw, TRUE);
           list = gtk_tree_view_new_with_model (model);
           gtk_widget_set_name (list, "yad-color-palette");
           g_object_unref (model);
@@ -205,14 +199,11 @@ color_create_widget (GtkWidget * dlg)
 void
 color_print_result (void)
 {
-  GdkColor c;
-  guint64 alpha;
+  GdkRGBA c;
   gchar *cs;
 
-  gtk_color_selection_get_current_color (GTK_COLOR_SELECTION (color), &c);
-  alpha = gtk_color_selection_get_current_alpha (GTK_COLOR_SELECTION (color));
-
-  cs = get_color (&c, alpha);
+  gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (color), &c);
+  cs = get_color (&c);
 
   if (cs)
     g_printf ("%s\n", cs);
