@@ -14,11 +14,9 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
-#include <glib/gprintf.h>
-
 #include "yad.h"
 
 static void
@@ -50,7 +48,7 @@ drop_data_cb (GtkWidget * w, GdkDragContext * dc, gint x, gint y,
                 action = g_strdup_printf (options.common_data.command, dstr);
               else
                 action = g_strdup_printf ("%s '%s'", options.common_data.command, dstr);
-              g_spawn_command_line_async (action, NULL);
+              run_command_async (action);
               g_free (action);
             }
           else
@@ -74,9 +72,13 @@ drop_data_cb (GtkWidget * w, GdkDragContext * dc, gint x, gint y,
               gchar *action, *arg;
 
               arg = g_shell_quote (dstr);
-              action = g_strdup_printf ("%s %s", options.common_data.command, arg);
+              if (g_strstr_len (options.common_data.command, -1, "%s"))
+                action = g_strdup_printf (options.common_data.command, arg);
+              else
+                action = g_strdup_printf ("%s %s", options.common_data.command, arg);
               g_free (arg);
-              g_spawn_command_line_async (action, NULL);
+              
+              run_command_async (action);
               g_free (action);
             }
           else
