@@ -14,21 +14,24 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
-#include <errno.h>
-
-#include <glib/gprintf.h>
-
 #include "yad.h"
 
 static GtkWidget *font;
 
+static void
+font_activated_cb (GtkFontChooser *w, gchar *fn, gpointer d)
+{
+  if (options.plug == -1)
+    yad_exit (options.data.def_resp);
+}
+
 static void
 realize_cb (GtkWidget * w, gpointer d)
 {
-  gtk_font_selection_set_font_name (GTK_FONT_SELECTION (w), options.common_data.font);
+  gtk_font_chooser_set_font (GTK_FONT_CHOOSER (w), options.common_data.font);
 }
 
 GtkWidget *
@@ -36,11 +39,13 @@ font_create_widget (GtkWidget * dlg)
 {
   GtkWidget *w;
 
-  w = font = gtk_font_selection_new ();
+  w = font = gtk_font_chooser_widget_new ();
   gtk_widget_set_name (w, "yad-font-widget");
 
   if (options.font_data.preview)
-    gtk_font_selection_set_preview_text (GTK_FONT_SELECTION (w), options.font_data.preview);
+    gtk_font_chooser_set_preview_text (GTK_FONT_CHOOSER (w), options.font_data.preview);
+
+  g_signal_connect (G_OBJECT (w), "font-activated", G_CALLBACK (font_activated_cb), NULL);
 
   /* font must be set after widget inserted in toplevel */
   if (options.common_data.font)
@@ -58,9 +63,9 @@ font_print_result (void)
       PangoFontFamily *family;
       gint size;
 
-      face = gtk_font_selection_get_face (GTK_FONT_SELECTION (font));
-      family = gtk_font_selection_get_family (GTK_FONT_SELECTION (font));
-      size = gtk_font_selection_get_size (GTK_FONT_SELECTION (font));
+      face = gtk_font_chooser_get_font_face (GTK_FONT_CHOOSER (font));
+      family = gtk_font_chooser_get_font_family (GTK_FONT_CHOOSER (font));
+      size = gtk_font_chooser_get_font_size (GTK_FONT_CHOOSER (font));
 
       if (options.common_data.quoted_output)
         {
@@ -68,7 +73,7 @@ font_print_result (void)
           gchar *q2 = g_shell_quote (pango_font_face_get_face_name (face));
 
           g_printf ("%s%s%s%s%d\n", q1, options.common_data.separator, q2,
-                    options.common_data.separator, size / 1000);
+                    options.common_data.separator, size / PANGO_SCALE);
 
           g_free (q1);
           g_free (q2);
@@ -76,12 +81,12 @@ font_print_result (void)
       else
         {
           g_printf ("%s%s%s%s%d\n", pango_font_family_get_name (family), options.common_data.separator,
-                    pango_font_face_get_face_name (face), options.common_data.separator, size / 1000);
+                    pango_font_face_get_face_name (face), options.common_data.separator, size / PANGO_SCALE);
         }
     }
   else
     {
-      gchar *fn = gtk_font_selection_get_font_name (GTK_FONT_SELECTION (font));
+      gchar *fn = gtk_font_chooser_get_font (GTK_FONT_CHOOSER (font));
 
       if (options.common_data.quoted_output)
         {
