@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include "yad.h"
@@ -45,6 +45,33 @@ typedef struct {
   gboolean in_term;
 } DEntry;
 
+static GdkPixbuf *
+scale_pixbuf (GdkPixbuf *pb)
+{
+  GdkPixbuf *res;
+
+  if (!pb)
+    return NULL;
+
+  if (options.common_data.icon_size > 0)
+    {
+      guint width, height;
+
+      width = gdk_pixbuf_get_width (pb);
+      height = gdk_pixbuf_get_height (pb);
+
+      if (options.common_data.icon_size != width || options.common_data.icon_size != height)
+        res = gdk_pixbuf_scale_simple (pb, options.common_data.icon_size,
+                                       options.common_data.icon_size, GDK_INTERP_BILINEAR);
+      else
+        res = g_object_ref (pb);
+    }
+  else
+    res = g_object_ref (pb);
+
+  return res;
+}
+
 static void
 select_cb (GObject * obj, gpointer data)
 {
@@ -107,14 +134,12 @@ activate_cb (GtkWidget * view, GtkTreePath * path, gpointer data)
     {
       if (in_term)
         {
-          gchar *tcmd;
-
-          tcmd = g_strdup_printf (options.icons_data.term, cmd);
-          g_spawn_command_line_async (tcmd, NULL);
+          gchar *tcmd = g_strdup_printf (options.icons_data.term, cmd);
+          run_command_async (tcmd);
           g_free (tcmd);
         }
       else
-        g_spawn_command_line_async (cmd, NULL);
+        run_command_async (cmd);
     }
 }
 
@@ -144,11 +169,12 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
       GError *err = NULL;
       GString *string = g_string_new (NULL);
 
-      while (channel->is_readable != TRUE);
+      while (channel->is_readable != TRUE)
+        usleep (100);
 
       do
         {
-          GdkPixbuf *pb;
+          GdkPixbuf *spb = NULL;
           gint status;
 
           do
@@ -207,21 +233,25 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
               }
             case COL_PIXBUF:
               if (options.icons_data.compact)
-                if (*string->str)
-                  pb = get_pixbuf (string->str, YAD_SMALL_ICON);
-                else
-                  pb = NULL;
+                {
+                  if (*string->str)
+                    spb = get_pixbuf (string->str, YAD_SMALL_ICON, TRUE);
+                }
               else
-                pb = get_pixbuf (string->str, YAD_BIG_ICON);
-              gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, pb, -1);
-              if (pb)
-                g_object_unref (pb);
+                {
+                  GdkPixbuf *pb = get_pixbuf (string->str, YAD_BIG_ICON, FALSE);
+                  if (pb)
+                    {
+                      spb = scale_pixbuf (pb);
+                      g_object_unref (pb);
+                    }
+                }
+              gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, spb, -1);
+              if (spb)
+                g_object_unref (spb);
               break;
             case COL_TERM:
-              if (strcasecmp (string->str, "true") == 0)
-                gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, TRUE, -1);
-              else
-                gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, FALSE, -1);
+              gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, get_bool_val (string->str), -1);
               break;
             }
 
@@ -315,7 +345,11 @@ parse_desktop_file (gchar * filename)
               gchar *url = g_key_file_get_string (kf, "Desktop Entry", "URL", NULL);
               if (url)
                 {
-                  ent->command = g_strdup_printf (settings.open_cmd, url);
+#ifndef STANDALONE
+                  ent->command = g_strdup_printf (g_settings_get_string (settings, "open-command"), url);
+#else
+                  ent->command = g_strdup_printf (OPEN_CMD, url);
+#endif
                   g_free (url);
                 }
             }
@@ -325,9 +359,14 @@ parse_desktop_file (gchar * filename)
           if (icon)
             {
               if (options.icons_data.compact)
-                ent->pixbuf = get_pixbuf (icon, YAD_SMALL_ICON);
+                ent->pixbuf = get_pixbuf (icon, YAD_SMALL_ICON, TRUE);
               else
-                ent->pixbuf = get_pixbuf (icon, YAD_BIG_ICON);
+                {
+                  GdkPixbuf *pb = get_pixbuf (icon, YAD_BIG_ICON, FALSE);
+                  ent->pixbuf = scale_pixbuf (pb);
+                  if (pb)
+                    g_object_unref (pb);
+                }
               g_free (icon);
             }
         }
@@ -392,14 +431,12 @@ read_dir ()
   g_dir_close (dir);
 }
 
-#ifdef HAVE_GIO
 static void
 dir_changed_cb (GFileMonitor *mon, GFile *file, GFile *ofile, GFileMonitorEvent ev, gpointer data)
 {
   if (ev == G_FILE_MONITOR_EVENT_DELETED || ev == G_FILE_MONITOR_EVENT_CREATED)
     read_dir ();
 }
-#endif
 
 GtkWidget *
 icons_create_widget (GtkWidget * dlg)
@@ -480,7 +517,6 @@ icons_create_widget (GtkWidget * dlg)
   else
     g_signal_connect (G_OBJECT (icon_view), "row-activated", G_CALLBACK (activate_cb), NULL);
 
-#ifdef HAVE_GIO
   /* start file monitor */
   if (options.icons_data.monitor && options.icons_data.directory)
     {
@@ -492,7 +528,6 @@ icons_create_widget (GtkWidget * dlg)
           g_object_unref (file);
         }
     }
-#endif
 
   gtk_container_add (GTK_CONTAINER (w), icon_view);
 
