@@ -14,43 +14,56 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <string.h>
 #include <stdlib.h>
 
-#include <glib/gprintf.h>
-
 #include "yad.h"
 
 static GtkWidget *list_view;
 
+static GHashTable *row_hash = NULL;
+
 static gint fore_col, back_col, font_col;
+static guint n_cols = 0;
 
 static gulong select_hndl = 0;
 
 static inline void
-yad_list_add_row (GtkListStore *m, GtkTreeIter *it)
+yad_list_add_row (GtkTreeStore *m, GtkTreeIter *it, gchar *row_id, gchar *par_id)
 {
-  if (options.list_data.add_on_top)
-    gtk_list_store_prepend (m, it);
-  else
-    gtk_list_store_append (m, it);
+  GtkTreePath *row_path;
+  GtkTreeIter *parent = NULL;
 
+  if (par_id && par_id[0])
+    {
+      GtkTreePath *par_path = g_hash_table_lookup (row_hash, par_id);
+      if (par_path)
+        {
+          GtkTreeIter pit;
+          if (gtk_tree_model_get_iter (GTK_TREE_MODEL (m), &pit, par_path))
+            parent = &pit;
+        }
+    }
+
+  if (options.list_data.add_on_top)
+    gtk_tree_store_prepend (m, it, parent);
+  else
+    gtk_tree_store_append (m, it, parent);
+
+  row_path = gtk_tree_model_get_path (GTK_TREE_MODEL (m), it);
+  if (row_id && row_id[0])
+    g_hash_table_insert (row_hash, row_id, row_path);
   if (options.common_data.tail)
-    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (list_view), gtk_tree_model_get_path (GTK_TREE_MODEL (m), it),
-                                  NULL, FALSE, 1.0, 1.0);
+    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (list_view), row_path, NULL, FALSE, 1.0, 1.0);
 }
 
 static gboolean
 list_activate_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-#if GTK_CHECK_VERSION(2,24,0)
   if (event->keyval == GDK_KEY_Return || event->keyval == GDK_KEY_KP_Enter)
-#else
-  if (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter)
-#endif
     {
       if (options.list_data.dclick_action)
         {
@@ -75,6 +88,33 @@ list_activate_cb (GtkWidget *widget, GdkEventKey *event, gpointer data)
   return FALSE;
 }
 
+/* custom tooltip signal handler for no-markup mode */
+static gboolean
+tooltip_cb (GtkWidget *w, gint x, gint y, gboolean mode, GtkTooltip *tip, gpointer data)
+{
+  gchar *str;
+  gint bx, by;
+  GtkTreePath *path;
+  GtkTreeIter iter;
+  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (w));
+
+  gtk_tree_view_convert_widget_to_bin_window_coords (GTK_TREE_VIEW (w), x, y, &bx, &by);
+  if (!gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (w), bx, by, &path, NULL, NULL, NULL))
+    return FALSE;
+
+  gtk_tree_model_get_iter (model, &iter, path);
+  gtk_tree_path_free (path);
+
+  gtk_tree_model_get (model, &iter, options.list_data.tooltip_column - 1, &str, -1);
+  if (str)
+    {
+      gtk_tooltip_set_text (tip, str);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 toggled_cb (GtkCellRendererToggle *cell, gchar *path_str, gpointer data)
 {
@@ -90,7 +130,7 @@ toggled_cb (GtkCellRendererToggle *cell, gchar *path_str, gpointer data)
 
   fixed ^= 1;
 
-  gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, fixed, -1);
+  gtk_tree_store_set (GTK_TREE_STORE (model), &iter, column, fixed, -1);
 
   gtk_tree_path_free (path);
 }
@@ -99,7 +139,7 @@ static gboolean
 runtoggle (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
 {
   gint col = GPOINTER_TO_INT (data);
-  gtk_list_store_set (GTK_LIST_STORE (model), iter, col, FALSE, -1);
+  gtk_tree_store_set (GTK_TREE_STORE (model), iter, col, FALSE, -1);
   return FALSE;
 }
 
@@ -116,7 +156,7 @@ rtoggled_cb (GtkCellRendererToggle *cell, gchar *path_str, gpointer data)
   gtk_tree_model_foreach (model, runtoggle, GINT_TO_POINTER (column));
 
   gtk_tree_model_get_iter (model, &iter, path);
-  gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, TRUE, -1);
+  gtk_tree_store_set (GTK_TREE_STORE (model), &iter, column, TRUE, -1);
 
   gtk_tree_path_free (path);
 }
@@ -135,11 +175,11 @@ cell_edited_cb (GtkCellRendererText *cell, const gchar *path_string, const gchar
   col = (YadColumn *) g_slist_nth_data (options.list_data.columns, column);
 
   if (col->type == YAD_COLUMN_NUM)
-    gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, g_ascii_strtoll (new_text, NULL, 10), -1);
+    gtk_tree_store_set (GTK_TREE_STORE (model), &iter, column, g_ascii_strtoll (new_text, NULL, 10), -1);
   else if (col->type == YAD_COLUMN_FLOAT)
-    gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, g_ascii_strtod (new_text, NULL), -1);
+    gtk_tree_store_set (GTK_TREE_STORE (model), &iter, column, g_ascii_strtod (new_text, NULL), -1);
   else
-    gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, new_text, -1);
+    gtk_tree_store_set (GTK_TREE_STORE (model), &iter, column, new_text, -1);
 
   gtk_tree_path_free (path);
 }
@@ -177,13 +217,13 @@ regex_search (GtkTreeModel *model, gint col, const gchar *key, GtkTreeIter *iter
 }
 
 static GtkTreeModel *
-create_model (gint n_columns)
+create_model ()
 {
-  GtkListStore *store;
+  GtkTreeStore *store;
   GType *ctypes;
   gint i;
 
-  ctypes = g_new0 (GType, n_columns);
+  ctypes = g_new0 (GType, n_cols);
 
   if (options.list_data.checkbox)
     {
@@ -196,7 +236,7 @@ create_model (gint n_columns)
       col->type = YAD_COLUMN_RADIO;
     }
 
-  for (i = 0; i < n_columns; i++)
+  for (i = 0; i < n_cols; i++)
     {
       YadColumn *col = (YadColumn *) g_slist_nth_data (options.list_data.columns, i);
 
@@ -235,7 +275,7 @@ create_model (gint n_columns)
         }
     }
 
-  store = gtk_list_store_newv (n_columns, ctypes);
+  store = gtk_tree_store_newv (n_cols, ctypes);
 
   return GTK_TREE_MODEL (store);
 }
@@ -273,13 +313,39 @@ size_col_format (GtkTreeViewColumn *col, GtkCellRenderer *cell, GtkTreeModel *mo
 }
 
 static void
-add_columns (gint n_columns)
+set_column_title (GtkTreeViewColumn *col, gchar *title)
+{
+  GtkWidget *lbl;
+  gchar **str;
+
+  str = g_strsplit (title, options.common_data.item_separator, 2);
+  lbl = gtk_label_new (NULL);
+
+  if (options.data.no_markup)
+    {
+      gtk_label_set_text (GTK_LABEL (lbl), str[0]);
+      if (str[1] || options.list_data.header_tips)
+        gtk_widget_set_tooltip_text (lbl, str[1] ? str[1] : str[0]);
+    }
+  else
+    {
+      gtk_label_set_markup (GTK_LABEL (lbl), str[0]);
+      if (str[1] || options.list_data.header_tips)
+        gtk_widget_set_tooltip_markup (lbl, str[1] ? str[1] : str[0]);
+    }
+
+  gtk_widget_show (lbl);
+  gtk_tree_view_column_set_widget (col, lbl);
+}
+
+static void
+add_columns ()
 {
   gint i;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
 
-  for (i = 0; i < n_columns; i++)
+  for (i = 0; i < n_cols; i++)
     {
       YadColumn *col = (YadColumn *) g_slist_nth_data (options.list_data.columns, i);
 
@@ -292,7 +358,8 @@ add_columns (gint n_columns)
         case YAD_COLUMN_CHECK:
         case YAD_COLUMN_RADIO:
           renderer = gtk_cell_renderer_toggle_new ();
-          column = gtk_tree_view_column_new_with_attributes (col->name, renderer, "active", i, NULL);
+          column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "active", i, NULL);
+          set_column_title (column, col->name);
           if (back_col != -1)
             gtk_tree_view_column_add_attribute (column, renderer, "cell-background", back_col);
           if (col->type == YAD_COLUMN_RADIO)
@@ -305,7 +372,8 @@ add_columns (gint n_columns)
           break;
         case YAD_COLUMN_IMAGE:
           renderer = gtk_cell_renderer_pixbuf_new ();
-          column = gtk_tree_view_column_new_with_attributes (col->name, renderer, "pixbuf", i, NULL);
+          column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "pixbuf", i, NULL);
+          set_column_title (column, col->name);
           if (back_col != -1)
             gtk_tree_view_column_add_attribute (column, renderer, "cell-background", back_col);
           break;
@@ -318,7 +386,8 @@ add_columns (gint n_columns)
               g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
               g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited_cb), NULL);
             }
-          column = gtk_tree_view_column_new_with_attributes (col->name, renderer, "text", i, NULL);
+          column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "text", i, NULL);
+          set_column_title (column, col->name);
           if (fore_col != -1)
             gtk_tree_view_column_add_attribute (column, renderer, "foreground", fore_col);
           if (back_col != -1)
@@ -334,7 +403,8 @@ add_columns (gint n_columns)
           break;
         case YAD_COLUMN_BAR:
           renderer = gtk_cell_renderer_progress_new ();
-          column = gtk_tree_view_column_new_with_attributes (col->name, renderer, "value", i, NULL);
+          column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "value", i, NULL);
+          set_column_title (column, col->name);
           if (back_col != -1)
             gtk_tree_view_column_add_attribute (column, renderer, "cell-background", back_col);
           gtk_tree_view_column_set_sort_column_id (column, i);
@@ -348,10 +418,12 @@ add_columns (gint n_columns)
               g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited_cb), NULL);
             }
           if (options.data.no_markup)
-            column = gtk_tree_view_column_new_with_attributes (col->name, renderer, "text", i, NULL);
+            column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "text", i, NULL);
           else
-            column = gtk_tree_view_column_new_with_attributes (col->name, renderer, "markup", i, NULL);
-          g_object_set (G_OBJECT (renderer), "ellipsize", col->ellipsize, NULL);
+            column = gtk_tree_view_column_new_with_attributes (NULL, renderer, "markup", i, NULL);
+          set_column_title (column, col->name);
+          if (col->ellipsize)
+            g_object_set (G_OBJECT (renderer), "ellipsize", options.list_data.ellipsize, NULL);
           if (col->wrap)
             {
               g_object_set (G_OBJECT (renderer), "wrap-width", options.list_data.wrap_width, NULL);
@@ -365,6 +437,9 @@ add_columns (gint n_columns)
             gtk_tree_view_column_add_attribute (column, renderer, "font", font_col);
           gtk_tree_view_column_set_sort_column_id (column, i);
           gtk_tree_view_column_set_resizable (column, TRUE);
+          
+          if (col->type == YAD_COLUMN_TIP)
+            options.list_data.tooltip_column = i + 1;
           break;
         }
       g_object_set_data (G_OBJECT (renderer), "column", GINT_TO_POINTER (i));
@@ -381,7 +456,7 @@ add_columns (gint n_columns)
 
   if (options.list_data.checkbox && !options.list_data.search_column)
     options.list_data.search_column += 1;
-  if (options.list_data.search_column <= n_columns)
+  if (options.list_data.search_column <= n_cols)
     {
       options.list_data.search_column -= 1;
       gtk_tree_view_set_search_column (GTK_TREE_VIEW (list_view), options.list_data.search_column);
@@ -398,14 +473,14 @@ cell_set_data (GtkTreeIter *it, guint num, gchar *data)
     {
     case YAD_COLUMN_CHECK:
     case YAD_COLUMN_RADIO:
-        gtk_list_store_set (GTK_LIST_STORE (model), it, num, (strcasecmp (data, "true") == 0), -1);
+        gtk_tree_store_set (GTK_TREE_STORE (model), it, num, get_bool_val (data), -1);
       break;
     case YAD_COLUMN_NUM:
     case YAD_COLUMN_SIZE:
-      gtk_list_store_set (GTK_LIST_STORE (model), it, num, g_ascii_strtoll (data, NULL, 10), -1);
+      gtk_tree_store_set (GTK_TREE_STORE (model), it, num, g_ascii_strtoll (data, NULL, 10), -1);
       break;
     case YAD_COLUMN_FLOAT:
-      gtk_list_store_set (GTK_LIST_STORE (model), it, num, g_ascii_strtod (data, NULL), -1);
+      gtk_tree_store_set (GTK_TREE_STORE (model), it, num, g_ascii_strtod (data, NULL), -1);
       break;
     case YAD_COLUMN_BAR:
       {
@@ -414,22 +489,22 @@ cell_set_data (GtkTreeIter *it, guint num, gchar *data)
           val = 0;
         if (val > 100)
           val = 100;
-        gtk_list_store_set (GTK_LIST_STORE (model), it, num, val, -1);
+        gtk_tree_store_set (GTK_TREE_STORE (model), it, num, val, -1);
         break;
       }
     case YAD_COLUMN_IMAGE:
       {
-        GdkPixbuf *pb = get_pixbuf (data, YAD_SMALL_ICON);
+        GdkPixbuf *pb = get_pixbuf (data, YAD_SMALL_ICON, FALSE);
         if (pb)
           {
-            gtk_list_store_set (GTK_LIST_STORE (model), it, num, pb, -1);
+            gtk_tree_store_set (GTK_TREE_STORE (model), it, num, pb, -1);
             g_object_unref (pb);
           }
         break;
       }
     default:
       if (data && *data)
-        gtk_list_store_set (GTK_LIST_STORE (model), it, num, data, -1);
+        gtk_tree_store_set (GTK_TREE_STORE (model), it, num, data, -1);
       break;
     }
 }
@@ -448,7 +523,7 @@ cell_get_data (GtkTreeIter *it, guint num)
       {
         gboolean bval;
         gtk_tree_model_get (model, it, num, &bval, -1);
-        data = g_strdup (bval ? "TRUE" : "FALSE");
+        data = g_strdup (print_bool_val (bval));
         break;
       }
     case YAD_COLUMN_NUM:
@@ -486,20 +561,21 @@ cell_get_data (GtkTreeIter *it, guint num)
 }
 
 static gboolean
-handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
+handle_stdin (GIOChannel *channel, GIOCondition condition, gpointer data)
 {
   static GtkTreeIter iter;
   static gint column_count = 0;
   static gint row_count = 0;
-  gint n_columns = GPOINTER_TO_INT (data);
   GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (list_view));
 
   if ((condition == G_IO_IN) || (condition == G_IO_IN + G_IO_HUP))
     {
       GError *err = NULL;
       GString *string = g_string_new (NULL);
+      gboolean node_added = FALSE;
 
-      while (channel->is_readable != TRUE);
+      while (channel->is_readable != TRUE)
+        usleep (100);
 
       do
         {
@@ -535,26 +611,56 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
               GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (list_view));
               if (select_hndl)
                 g_signal_handler_block (G_OBJECT (sel), select_hndl);
-              gtk_list_store_clear (GTK_LIST_STORE (model));
+              gtk_tree_store_clear (GTK_TREE_STORE (model));
               row_count = column_count = 0;
+              if (row_hash)
+                g_hash_table_remove_all (row_hash);
               if (select_hndl)
                 g_signal_handler_unblock (G_OBJECT (sel), select_hndl);
               continue;
             }
 
           if (row_count == 0 && column_count == 0)
-            yad_list_add_row (GTK_LIST_STORE (model), &iter);
-          else if (column_count == n_columns)
+            {
+              if (options.list_data.tree_mode)
+                {
+                  if (!node_added)
+                    {
+                      gchar **ids = g_strsplit (string->str, ":", 2);
+                      yad_list_add_row (GTK_TREE_STORE (model), &iter, ids[0], ids[1]);
+                      node_added = TRUE;
+                      continue;
+                    }
+                  else
+                    node_added = FALSE;
+                }
+              else
+                yad_list_add_row (GTK_TREE_STORE (model), &iter, NULL, NULL);
+            }
+          else if (column_count == n_cols)
             {
               /* We're starting a new row */
+              if (options.list_data.tree_mode)
+                {
+                  if (!node_added)
+                    {
+                      gchar **ids = g_strsplit (string->str, ":", 2);
+                      yad_list_add_row (GTK_TREE_STORE (model), &iter, ids[0], ids[1]);
+                      node_added = TRUE;
+                      continue;
+                    }
+                  else
+                    node_added = FALSE;
+                }
+              else
+                yad_list_add_row (GTK_TREE_STORE (model), &iter, NULL, NULL);
               column_count = 0;
               row_count++;
               if (options.list_data.limit && row_count >= options.list_data.limit)
                 {
                   gtk_tree_model_get_iter_first (model, &iter);
-                  gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+                  gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
                 }
-              yad_list_add_row (GTK_LIST_STORE (model), &iter);
             }
 
           cell_set_data (&iter, column_count, string->str);
@@ -564,6 +670,9 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
       g_string_free (string, TRUE);
     }
 
+  if (options.list_data.tree_expanded)
+    gtk_tree_view_expand_all (GTK_TREE_VIEW (list_view));
+
   if ((condition != G_IO_IN) && (condition != G_IO_IN + G_IO_HUP))
     {
       g_io_channel_shutdown (channel, TRUE, NULL);
@@ -574,10 +683,10 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
 }
 
 static void
-fill_data (gint n_columns)
+fill_data ()
 {
   GtkTreeIter iter;
-  GtkListStore *model = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (list_view)));
+  GtkTreeStore *model = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (list_view)));
   GIOChannel *channel;
 
   if (options.extra_data && *options.extra_data)
@@ -591,8 +700,15 @@ fill_data (gint n_columns)
         {
           gint j;
 
-          yad_list_add_row (model, &iter);
-          for (j = 0; j < n_columns; j++, i++)
+          if (options.list_data.tree_mode)
+            {
+              gchar **ids = g_strsplit (args[i], ":", 2);
+              yad_list_add_row (model, &iter, ids[0], ids[1]);
+              i++;
+            }
+          else
+            yad_list_add_row (model, &iter, NULL, NULL);
+          for (j = 0; j < n_cols; j++, i++)
             {
               if (args[i] == NULL)
                 break;
@@ -609,12 +725,40 @@ fill_data (gint n_columns)
       channel = g_io_channel_unix_new (0);
       g_io_channel_set_encoding (channel, NULL, NULL);
       g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, NULL);
-      g_io_add_watch (channel, G_IO_IN | G_IO_HUP, handle_stdin, GINT_TO_POINTER (n_columns));
+      g_io_add_watch (channel, G_IO_IN | G_IO_HUP, handle_stdin, NULL);
     }
 }
 
+static gchar *
+get_data_as_string (GtkTreeIter *iter)
+{
+  GString *str;
+  gchar *res;
+  guint i;
+
+  str = g_string_new (NULL);
+
+  for (i = 0; i < n_cols; i++)
+    {
+      gchar *val = cell_get_data (iter, i);
+      if (val)
+        {
+          g_string_append_printf (str, "%s ", val);
+          g_free (val);
+        }
+    }
+
+  str->str[str->len-1] = '\0';
+  res = str->str;
+  g_string_free (str, FALSE);
+
+  return res;
+}
+
+static void edit_row_cb (GtkMenuItem *item, gpointer data);
+
 static void
-double_click_cb (GtkTreeView * view, GtkTreePath * path, GtkTreeViewColumn * column, gpointer data)
+double_click_cb (GtkTreeView *view, GtkTreePath *path, GtkTreeViewColumn *column, gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter iter;
@@ -623,28 +767,12 @@ double_click_cb (GtkTreeView * view, GtkTreePath * path, GtkTreeViewColumn * col
 
   if (options.list_data.dclick_action)
     {
-      gchar *cmd;
-      GString *args;
-      guint n_cols;
-
-      args = g_string_new ("");
-
-      n_cols = gtk_tree_model_get_n_columns (model);
+      gchar *cmd, *args = NULL;
 
       if (gtk_tree_model_get_iter (model, &iter, path))
-        {
-          gint i;
-
-          for (i = 0; i < n_cols; i++)
-            {
-              gchar *val = cell_get_data (&iter, i);
-              if (val)
-                {
-                  g_string_append_printf (args, " %s", val);
-                  g_free (val);
-                }
-            }
-        }
+        args = get_data_as_string (&iter);
+      else
+        args = g_strdup ("");
 
       if (g_strstr_len (options.list_data.dclick_action, -1, "%s"))
         {
@@ -652,18 +780,18 @@ double_click_cb (GtkTreeView * view, GtkTreePath * path, GtkTreeViewColumn * col
 
           if (!regex)
             regex = g_regex_new ("\%s", G_REGEX_OPTIMIZE, 0, NULL);
-          cmd = g_regex_replace_literal (regex, options.list_data.dclick_action, -1, 0, args->str, 0, NULL);
+          cmd = g_regex_replace_literal (regex, options.list_data.dclick_action, -1, 0, args, 0, NULL);
         }
       else
-        cmd = g_strdup_printf ("%s %s", options.list_data.dclick_action, args->str);
-      g_string_free (args, TRUE);
+        cmd = g_strdup_printf ("%s %s", options.list_data.dclick_action, args);
+      g_free (args);
 
       if (cmd[0] == '@')
         {
           gchar *data = NULL;
           gint exit;
 
-          g_spawn_command_line_sync (cmd + 1, &data, NULL, &exit, NULL);
+          exit = run_command_sync (cmd + 1, &data);
           if (exit == 0)
             {
               gint i;
@@ -681,10 +809,12 @@ double_click_cb (GtkTreeView * view, GtkTreePath * path, GtkTreeViewColumn * col
           g_free (data);
         }
       else
-        g_spawn_command_line_async (cmd, NULL);
+        run_command_async (cmd);
 
       g_free (cmd);
     }
+  else if (options.common_data.editable && options.list_data.row_action)
+    edit_row_cb (NULL, NULL);
   else
     {
       if (options.list_data.checkbox)
@@ -695,7 +825,7 @@ double_click_cb (GtkTreeView * view, GtkTreePath * path, GtkTreeViewColumn * col
 
               gtk_tree_model_get (model, &iter, 0, &chk, -1);
               chk = !chk;
-              gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, chk, -1);
+              gtk_tree_store_set (GTK_TREE_STORE (model), &iter, 0, chk, -1);
             }
         }
       else if (options.list_data.radiobox)
@@ -703,7 +833,7 @@ double_click_cb (GtkTreeView * view, GtkTreePath * path, GtkTreeViewColumn * col
           if (gtk_tree_model_get_iter (model, &iter, path))
             {
               gtk_tree_model_foreach (model, runtoggle, GINT_TO_POINTER (0));
-              gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, TRUE, -1);
+              gtk_tree_store_set (GTK_TREE_STORE (model), &iter, 0, TRUE, -1);
             }
         }
       else if (options.plug == -1)
@@ -716,25 +846,14 @@ select_cb (GtkTreeSelection *sel, gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter iter;
-  gchar *cmd;
-  GString *args;
-  guint i, n_cols;
+  gchar *cmd, *args;
 
   if (!gtk_tree_selection_get_selected (sel, &model, &iter))
     return;
 
-  args = g_string_new ("");
-  n_cols = gtk_tree_model_get_n_columns (model);
-
-  for (i = 0; i < n_cols; i++)
-    {
-      gchar *val = cell_get_data (&iter, i);
-      if (val)
-        {
-          g_string_append_printf (args, " %s", val);
-          g_free (val);
-        }
-    }
+  args = get_data_as_string (&iter);
+  if (!args)
+    args = g_strdup ("");
 
   if (g_strstr_len (options.list_data.select_action, -1, "%s"))
     {
@@ -742,27 +861,39 @@ select_cb (GtkTreeSelection *sel, gpointer data)
 
       if (!regex)
         regex = g_regex_new ("\%s", G_REGEX_OPTIMIZE, 0, NULL);
-      cmd = g_regex_replace_literal (regex, options.list_data.select_action, -1, 0, args->str, 0, NULL);
+      cmd = g_regex_replace_literal (regex, options.list_data.select_action, -1, 0, args, 0, NULL);
     }
   else
-    cmd = g_strdup_printf ("%s %s", options.list_data.select_action, args->str);
-  g_string_free (args, TRUE);
+    cmd = g_strdup_printf ("%s %s", options.list_data.select_action, args);
+  g_free (args);
 
-  g_spawn_command_line_async (cmd, NULL);
+  run_command_async (cmd);
 
   g_free (cmd);
 }
 
 static void
-add_row_cb (GtkMenuItem * item, gpointer data)
+add_row_cb (GtkMenuItem *item, gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter iter;
+  gchar *cmd;
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (list_view));
-  yad_list_add_row (GTK_LIST_STORE (model), &iter);
+  if (g_object_get_data (G_OBJECT (item), "child") != NULL)
+    {
+      GtkTreeIter parent;
+      GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (list_view));
 
-  if (options.list_data.add_action)
+      if (gtk_tree_selection_get_selected (sel, NULL, &parent))
+        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, &parent);
+      else
+        gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
+    }
+  else
+    gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
+
+  if (options.list_data.row_action)
     {
       gchar *out = NULL;
       gint exit;
@@ -773,10 +904,12 @@ add_row_cb (GtkMenuItem * item, gpointer data)
         gtk_main_iteration ();
 
       /* run command */
-      g_spawn_command_line_sync (options.list_data.add_action, &out, NULL, &exit, NULL);
+      cmd = g_strdup_printf ("%s add", options.list_data.row_action);
+      exit = run_command_sync (cmd, &out);
+      g_free (cmd);
       if (exit == 0)
         {
-          guint i, n_cols = gtk_tree_model_get_n_columns (model);
+          guint i;
           gchar **lines = g_strsplit (out, "\n", 0);
 
           for (i = 0; i < n_cols; i++)
@@ -793,33 +926,99 @@ add_row_cb (GtkMenuItem * item, gpointer data)
 }
 
 static void
-del_row_cb (GtkMenuItem * item, gpointer data)
+edit_row_cb (GtkMenuItem *item, gpointer data)
+{
+  GtkTreeIter iter;
+  GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (list_view));
+
+  if (!gtk_tree_selection_get_selected (sel, NULL, &iter))
+    return;
+
+  if (options.list_data.row_action)
+    {
+      gchar *cmd, *args, *out = NULL;
+      gint exit;
+
+      /* hide menu first */
+      gtk_menu_popdown (GTK_MENU (data));
+      while (gtk_events_pending ())
+        gtk_main_iteration ();
+
+      /* run command */
+      args = get_data_as_string (&iter);
+      cmd = g_strdup_printf ("%s edit %s", options.list_data.row_action, args);
+      g_free (args);
+      exit = run_command_sync (cmd, &out);
+      g_free (cmd);
+      if (exit == 0)
+        {
+          guint i;
+          gchar **lines = g_strsplit (out, "\n", 0);
+
+          for (i = 0; i < n_cols; i++)
+            {
+              if (lines[i] == NULL)
+                break;
+
+              cell_set_data (&iter, i, lines[i]);
+            }
+          g_strfreev (lines);
+        }
+      g_free (out);
+    }
+}
+
+static void
+del_row_cb (GtkMenuItem *item, gpointer data)
 {
   GtkTreeIter iter;
   GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (list_view));
   GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (list_view));
 
   if (gtk_tree_selection_get_selected (sel, NULL, &iter))
-    gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+    {
+      if (options.list_data.row_action)
+        {
+          gchar *cmd, *args;
+          gint exit;
+
+          /* hide menu first */
+          gtk_menu_popdown (GTK_MENU (data));
+          while (gtk_events_pending ())
+            gtk_main_iteration ();
+
+          /* run command */
+          args = get_data_as_string (&iter);
+          cmd = g_strdup_printf ("%s del %s", options.list_data.row_action, args);
+          g_free (args);
+          exit = run_command_sync (cmd, NULL);
+          g_free (cmd);
+          if (exit == 0)
+            gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
+        }
+      else
+        gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
+    }
 }
 
 static void
-copy_row_cb (GtkMenuItem * item, gpointer data)
+copy_row_cb (GtkMenuItem *item, gpointer data)
 {
-  GtkTreeIter iter, new_iter;
+  GtkTreeIter iter;
   GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (list_view));
   GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (list_view));
 
   if (gtk_tree_selection_get_selected (sel, NULL, &iter))
     {
-      gint i, n_columns;
+      GtkTreeIter new_iter, parent;
+      gint i;
 
-      /* decrease by 1 due to last ellipsize column */
-      n_columns = gtk_tree_model_get_n_columns (model);
+      if (gtk_tree_model_iter_parent (model, &parent, &iter))
+        gtk_tree_store_insert_after (GTK_TREE_STORE (model), &new_iter, &parent, &iter);
+      else
+        gtk_tree_store_insert_after (GTK_TREE_STORE (model), &new_iter, NULL, &iter);
 
-      gtk_list_store_insert_after (GTK_LIST_STORE (model), &new_iter, &iter);
-
-      for (i = 0; i < n_columns; i++)
+      for (i = 0; i < n_cols; i++)
         {
           GdkPixbuf *pb;
           gchar *tv;
@@ -833,25 +1032,25 @@ copy_row_cb (GtkMenuItem * item, gpointer data)
             case YAD_COLUMN_CHECK:
             case YAD_COLUMN_RADIO:
               gtk_tree_model_get (model, &iter, i, &bv, -1);
-              gtk_list_store_set (GTK_LIST_STORE (model), &new_iter, i, bv, -1);
+              gtk_tree_store_set (GTK_TREE_STORE (model), &new_iter, i, bv, -1);
               break;
             case YAD_COLUMN_NUM:
             case YAD_COLUMN_SIZE:
             case YAD_COLUMN_BAR:
               gtk_tree_model_get (model, &iter, i, &iv, -1);
-              gtk_list_store_set (GTK_LIST_STORE (model), &new_iter, i, iv, -1);
+              gtk_tree_store_set (GTK_TREE_STORE (model), &new_iter, i, iv, -1);
               break;
             case YAD_COLUMN_FLOAT:
               gtk_tree_model_get (model, &iter, i, &fv, -1);
-              gtk_list_store_set (GTK_LIST_STORE (model), &new_iter, i, fv, -1);
+              gtk_tree_store_set (GTK_TREE_STORE (model), &new_iter, i, fv, -1);
               break;
             case YAD_COLUMN_IMAGE:
               gtk_tree_model_get (model, &iter, i, &pb, -1);
-              gtk_list_store_set (GTK_LIST_STORE (model), &new_iter, i, pb, -1);
+              gtk_tree_store_set (GTK_TREE_STORE (model), &new_iter, i, g_object_ref (pb), -1);
               break;
             default:
               gtk_tree_model_get (model, &iter, i, &tv, -1);
-              gtk_list_store_set (GTK_LIST_STORE (model), &new_iter, i, tv, -1);
+              gtk_tree_store_set (GTK_TREE_STORE (model), &new_iter, i, g_strdup (tv), -1);
               break;
             }
         }
@@ -859,7 +1058,7 @@ copy_row_cb (GtkMenuItem * item, gpointer data)
 }
 
 static gboolean
-popup_menu_cb (GtkWidget * w, GdkEventButton * ev, gpointer data)
+popup_menu_cb (GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
   static GtkWidget *menu = NULL;
   if (ev->button == 3)
@@ -869,37 +1068,49 @@ popup_menu_cb (GtkWidget * w, GdkEventButton * ev, gpointer data)
       if (menu == NULL)
         {
           menu = gtk_menu_new ();
+          gtk_menu_set_reserve_toggle_size (GTK_MENU (menu), FALSE);
 
-          item = gtk_image_menu_item_new_with_label (_("Add row"));
-          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
-                                         gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU));
+          item = gtk_menu_item_new_with_label (_("Add row"));
           gtk_widget_show (item);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
           g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (add_row_cb), menu);
 
-          item = gtk_image_menu_item_new_with_label (_("Delete row"));
-          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
-                                         gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU));
+          if (options.list_data.tree_mode)
+            {
+              item = gtk_menu_item_new_with_label (_("Add child row"));
+              gtk_widget_show (item);
+              gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+              g_object_set_data (G_OBJECT (item), "child", "1");
+              g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (add_row_cb), menu);
+            }
+
+          item = gtk_menu_item_new_with_label (_("Delete row"));
           gtk_widget_show (item);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
           g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (del_row_cb), menu);
 
-          item = gtk_image_menu_item_new_with_label (_("Duplicate row"));
-          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
-                                         gtk_image_new_from_stock (GTK_STOCK_COPY, GTK_ICON_SIZE_MENU));
+          if (options.list_data.row_action)
+            {
+              item = gtk_menu_item_new_with_label (_("Edit row"));
+              gtk_widget_show (item);
+              gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+              g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (del_row_cb), menu);
+            }
+
+          item = gtk_menu_item_new_with_label (_("Duplicate row"));
           gtk_widget_show (item);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
           g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (copy_row_cb), menu);
 
           gtk_widget_show (menu);
         }
-      gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, ev->button, ev->time);
+      gtk_menu_popup_at_pointer (GTK_MENU (menu), NULL);
     }
   return FALSE;
 }
 
 static gboolean
-row_sep_func (GtkTreeModel * m, GtkTreeIter * it, gpointer data)
+row_sep_func (GtkTreeModel *m, GtkTreeIter *it, gpointer data)
 {
   gchar *name;
 
@@ -1017,22 +1228,29 @@ parse_cols_props ()
 }
 
 GtkWidget *
-list_create_widget (GtkWidget * dlg)
+list_create_widget (GtkWidget *dlg)
 {
   GtkWidget *w;
   GtkTreeModel *model;
-  gint n_columns;
 
   fore_col = back_col = font_col = -1;
 
-  n_columns = g_slist_length (options.list_data.columns);
+  if (options.debug)
+    {
+      if (options.list_data.checkbox || options.list_data.radiobox)
+        g_printerr (_("WARNING: You are use --checklist or --radiolist option. Those options obsoleted and will be removed in the future\n"));
+    }
 
-  if (n_columns == 0)
+  n_cols = g_slist_length (options.list_data.columns);
+  if (n_cols == 0)
     {
       g_printerr (_("No column titles specified for List dialog.\n"));
       return NULL;
     }
 
+  if (options.list_data.tree_mode)
+    row_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) gtk_tree_path_free);
+
   parse_cols_props ();
 
   /* create widget */
@@ -1040,19 +1258,18 @@ list_create_widget (GtkWidget * dlg)
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (w), GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (w), options.hscroll_policy, options.vscroll_policy);
 
-  model = create_model (n_columns);
+  model = create_model ();
 
   list_view = gtk_tree_view_new_with_model (model);
   gtk_widget_set_name (list_view, "yad-list-widget");
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (list_view), !options.list_data.no_headers);
-  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (list_view), options.list_data.rules_hint);
   gtk_tree_view_set_grid_lines (GTK_TREE_VIEW (list_view), options.list_data.grid_lines);
   gtk_tree_view_set_reorderable (GTK_TREE_VIEW (list_view), options.common_data.editable);
   g_object_unref (model);
 
   gtk_container_add (GTK_CONTAINER (w), list_view);
 
-  add_columns (n_columns);
+  add_columns ();
 
   /* add popup menu */
   if (options.common_data.editable)
@@ -1060,7 +1277,15 @@ list_create_widget (GtkWidget * dlg)
 
   /* add tooltip column */
   if (options.list_data.tooltip_column > 0)
-    gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (list_view), options.list_data.tooltip_column - 1);
+    {
+      if (options.list_data.simple_tips || options.data.no_markup)
+        {
+          gtk_widget_set_has_tooltip (list_view, TRUE);
+          g_signal_connect (G_OBJECT (list_view), "query-tooltip", G_CALLBACK (tooltip_cb), NULL);
+        }
+      else
+        gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (list_view), options.list_data.tooltip_column - 1);
+    }
 
   /* set search function for regex search */
   if (options.list_data.search_column != -1 && options.list_data.regex_search)
@@ -1097,13 +1322,16 @@ list_create_widget (GtkWidget * dlg)
     }
 
   /* load data */
-  fill_data (n_columns);
+  fill_data ();
+
+  if (options.list_data.tree_expanded)
+    gtk_tree_view_expand_all (GTK_TREE_VIEW (list_view));
 
   return w;
 }
 
 static void
-print_col (GtkTreeModel * model, GtkTreeIter * iter, gint num)
+print_col (GtkTreeModel *model, GtkTreeIter *iter, gint num)
 {
   YadColumn *col = (YadColumn *) g_slist_nth_data (options.list_data.columns, num);
 
@@ -1119,9 +1347,9 @@ print_col (GtkTreeModel * model, GtkTreeIter * iter, gint num)
         gboolean bval;
         gtk_tree_model_get (model, iter, num, &bval, -1);
         if (options.common_data.quoted_output)
-          g_printf ("'%s'", bval ? "TRUE" : "FALSE");
+          g_printf ("'%s'", print_bool_val (bval));
         else
-          g_printf ("%s", bval ? "TRUE" : "FALSE");
+          g_printf ("%s", print_bool_val (bval));
         break;
       }
     case YAD_COLUMN_NUM:
@@ -1161,7 +1389,7 @@ print_col (GtkTreeModel * model, GtkTreeIter * iter, gint num)
             g_free (buf);
           }
         else
-          g_printf ("%s", val);
+          g_printf ("%s", val ? val : "");
         break;
       }
     }
@@ -1169,12 +1397,11 @@ print_col (GtkTreeModel * model, GtkTreeIter * iter, gint num)
 }
 
 static void
-print_selected (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
+print_selected (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
 {
-  gint i, n_cols, col;
+  gint i,col;
 
   col = options.list_data.print_column;
-  n_cols = gtk_tree_model_get_n_columns (model);
 
   if (col && col <= n_cols)
     print_col (model, iter, col - 1);
@@ -1187,18 +1414,20 @@ print_selected (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gp
 }
 
 static void
-print_all (GtkTreeModel * model)
+print_all (GtkTreeModel *model, GtkTreeIter *parent)
 {
   GtkTreeIter iter;
-  gint i, n_cols = gtk_tree_model_get_n_columns (model);
+  gint i;
 
-  if (gtk_tree_model_get_iter_first (model, &iter))
+  if (gtk_tree_model_iter_children (model, &iter, parent))
     {
       do
         {
           for (i = 0; i < n_cols; i++)
             print_col (model, &iter, i);
           g_printf ("\n");
+          /* print children */
+          print_all (model, &iter);
         }
       while (gtk_tree_model_iter_next (model, &iter));
     }
@@ -1214,7 +1443,7 @@ list_print_result (void)
 
   if (options.list_data.print_all)
     {
-      print_all (model);
+      print_all (model, NULL);
       return;
     }
 
@@ -1253,7 +1482,7 @@ list_print_result (void)
                   if (chk)
                     {
                       gint i;
-                      for (i = 0; i < gtk_tree_model_get_n_columns (model); i++)
+                      for (i = 0; i < n_cols; i++)
                         print_col (model, &iter, i);
                       g_printf ("\n");
                     }
