@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <sys/types.h>
@@ -34,16 +34,23 @@
 #include "yad.h"
 
 YadOptions options;
+GtkIconTheme *yad_icon_theme;
+
+#ifndef STANDALONE
+GSettings *settings;
+#endif
+
+GdkPixbuf *big_fallback_image = NULL;
+GdkPixbuf *small_fallback_image = NULL;
+
 static GtkWidget *dialog = NULL;
+static GtkWidget *text = NULL;
 
 static gint ret = YAD_RESPONSE_ESC;
 
-#if GTK_CHECK_VERSION(3,0,0)
-static GtkCssProvider *css = NULL;
-#endif
+static gboolean is_x11 = FALSE;
 
 YadNTabs *tabs;
-gint t_sem;
 
 #ifndef G_OS_WIN32
 static void
@@ -73,23 +80,14 @@ keys_cb (GtkWidget *w, GdkEventKey *ev, gpointer d)
 
   switch (ev->keyval)
     {
-#if GTK_CHECK_VERSION(2,24,0)
     case GDK_KEY_Escape:
-#else
-    case GDK_Escape:
-#endif
       if (options.data.escape_ok)
           yad_exit (options.data.def_resp);
       else if (!options.data.no_escape)
          yad_exit (YAD_RESPONSE_ESC);
       return TRUE;
-#if GTK_CHECK_VERSION(2,24,0)
     case GDK_KEY_Return:
     case GDK_KEY_KP_Enter:
-#else
-    case GDK_Return:
-    case GDK_KP_Enter:
-#endif
       if (ev->state & GDK_CONTROL_MASK)
         {
           yad_exit (options.data.def_resp);
@@ -103,7 +101,7 @@ static void
 btn_cb (GtkWidget *b, gchar *cmd)
 {
   if (cmd)
-    g_spawn_command_line_async (cmd, NULL);
+    run_command_async (cmd);
   else
     {
       gint resp = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (b), "resp"));
@@ -127,7 +125,11 @@ timeout_cb (gpointer data)
     {
       gdouble percent = ((gdouble) options.data.timeout - count) / (gdouble) options.data.timeout;
       gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (w), percent);
-      if (settings.show_remain)
+#ifndef STANDALONE
+      if (g_settings_get_boolean (settings, "show-remain"))
+#else
+      if (SHOW_REMAIN)
+#endif
         {
           gchar *lbl = g_strdup_printf (_("%d sec"), options.data.timeout - count);
           gtk_progress_bar_set_text (GTK_PROGRESS_BAR (w), lbl);
@@ -148,21 +150,10 @@ unfocus_cb (GtkWidget *w, GdkEventFocus *ev, gpointer d)
   return FALSE;
 }
 
-#if !GTK_CHECK_VERSION(3,0,0)
-static void
-text_size_allocate_cb (GtkWidget * w, GtkAllocation * al, gpointer data)
-{
-  PangoLayout *pl = gtk_label_get_layout (GTK_LABEL (w));
-
-  if (pango_layout_is_wrapped (pl))
-    gtk_widget_set_size_request (w, al->width, -1);
-}
-#endif
-
 void
 yad_exit (gint id)
 {
-  if (options.mode == YAD_MODE_FILE)
+  if ((options.mode == YAD_MODE_FILE) && !(id & 1))
     {
       /* show custom confirmation dialog */
       if (!file_confirm_overwrite (dialog))
@@ -176,22 +167,23 @@ yad_exit (gint id)
 static GtkWidget *
 create_layout (GtkWidget *dlg)
 {
-  GtkWidget *image, *text, *mw, *imw, *layout, *exp, *box;
+  GtkWidget *image, *mw, *imw, *layout, *exp, *box;
 
-  layout = image = text = mw = exp = NULL;
+  layout = image = mw = exp = NULL;
 
   /* create image */
   if (options.data.dialog_image)
     {
       GdkPixbuf *pb = NULL;
 
-      pb = get_pixbuf (options.data.dialog_image, YAD_BIG_ICON);
+      pb = get_pixbuf (options.data.dialog_image, YAD_BIG_ICON, FALSE);
       image = gtk_image_new_from_pixbuf (pb);
       if (pb)
         g_object_unref (pb);
 
       gtk_widget_set_name (image, "yad-dialog-image");
-      gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+      gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
+      gtk_widget_set_valign (image, GTK_ALIGN_START);
     }
 
   /* create text label */
@@ -213,30 +205,18 @@ create_layout (GtkWidget *dlg)
           gtk_label_set_line_wrap (GTK_LABEL (text), TRUE);
           gtk_label_set_selectable (GTK_LABEL (text), options.data.selectable_labels);
           gtk_label_set_justify (GTK_LABEL (text), options.data.text_align);
-          gtk_widget_set_state (text, GTK_STATE_NORMAL);
-          switch (options.data.text_align)
-            {
-            case GTK_JUSTIFY_LEFT:
-            case GTK_JUSTIFY_FILL:
-              gtk_misc_set_alignment (GTK_MISC (text), 0.0, 0.5);
-              break;
-            case GTK_JUSTIFY_CENTER:
-              gtk_misc_set_alignment (GTK_MISC (text), 0.5, 0.5);
-              break;
-            case GTK_JUSTIFY_RIGHT:
-              gtk_misc_set_alignment (GTK_MISC (text), 1.0, 0.5);
-              break;
-            }
-#if !GTK_CHECK_VERSION(3,0,0)
-          if (!options.data.fixed)
-            g_signal_connect (G_OBJECT (text), "size-allocate", G_CALLBACK (text_size_allocate_cb), NULL);
-#endif
+          gtk_widget_set_state_flags (text, GTK_STATE_NORMAL, FALSE);
+          gtk_label_set_xalign (GTK_LABEL (text), options.data.text_align);
+          gtk_widget_set_can_focus (text, FALSE);
         }
     }
 
   /* create main widget */
   switch (options.mode)
     {
+    case YAD_MODE_APP:
+      mw = app_create_widget (dlg);
+      break;
     case YAD_MODE_CALENDAR:
       mw = calendar_create_widget (dlg);
       break;
@@ -266,9 +246,6 @@ create_layout (GtkWidget *dlg)
     case YAD_MODE_LIST:
       mw = list_create_widget (dlg);
       break;
-    case YAD_MODE_MULTI_PROGRESS:
-      mw = multi_progress_create_widget (dlg);
-      break;
     case YAD_MODE_NOTEBOOK:
       if (options.plug == -1)
         mw = notebook_create_widget (dlg);
@@ -308,44 +285,17 @@ create_layout (GtkWidget *dlg)
     imw = mw;
 
   /* create layout */
-  if (options.data.image_on_top)
-    {
-#if !GTK_CHECK_VERSION(3,0,0)
-      layout = gtk_vbox_new (FALSE, 5);
-      box = gtk_hbox_new (FALSE, 5);
-#else
-      layout = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
-      box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
-#endif
+  layout = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
 
-      if (image)
-        gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 2);
-      if (text)
-        gtk_box_pack_start (GTK_BOX (box), text, TRUE, TRUE, 2);
+  if (image)
+    gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 2);
+  if (text)
+    gtk_box_pack_start (GTK_BOX (box), text, TRUE, TRUE, 2);
 
-      gtk_box_pack_start (GTK_BOX (layout), box, FALSE, FALSE, 0);
-      if (imw)
-        gtk_box_pack_start (GTK_BOX (layout), imw, TRUE, TRUE, 0);
-    }
-  else
-    {
-#if !GTK_CHECK_VERSION(3,0,0)
-      layout = gtk_hbox_new (FALSE, 5);
-      box = gtk_vbox_new (FALSE, 5);
-#else
-      layout = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
-      box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
-#endif
-
-      if (text)
-        gtk_box_pack_start (GTK_BOX (box), text, FALSE, FALSE, 0);
-      if (imw)
-        gtk_box_pack_start (GTK_BOX (box), imw, TRUE, TRUE, 0);
-
-      if (image)
-        gtk_box_pack_start (GTK_BOX (layout), image, FALSE, FALSE, 0);
-      gtk_box_pack_start (GTK_BOX (layout), box, TRUE, TRUE, 0);
-    }
+  gtk_box_pack_start (GTK_BOX (layout), box, FALSE, FALSE, 0);
+  if (imw)
+    gtk_box_pack_start (GTK_BOX (layout), imw, TRUE, TRUE, 0);
 
   if (options.mode == YAD_MODE_DND)
     dnd_init (layout);
@@ -353,6 +303,38 @@ create_layout (GtkWidget *dlg)
   return layout;
 }
 
+static void
+realize_cb (GtkWidget *dlg, gpointer d)
+{
+  gint cw, ch;
+  /* get current window size for gtk_window_resize */
+  gtk_window_get_size (GTK_WINDOW (dlg), &cw, &ch);
+  if (options.data.width == -1)
+    options.data.width = cw;
+  if (options.data.height == -1)
+    options.data.height = ch;
+
+  gtk_window_resize (GTK_WINDOW (dlg), options.data.width, options.data.height);
+  gtk_window_set_resizable (GTK_WINDOW (dlg), !options.data.fixed);
+
+  if (options.data.use_posx || options.data.use_posy)
+    {
+      gint ww, wh, sw, sh;
+      gtk_window_get_size (GTK_WINDOW (dlg), &ww, &wh);
+      gdk_window_get_geometry (gdk_get_default_root_window (), NULL, NULL, &sw, &sh);
+      /* place window to specified coordinates */
+      if (!options.data.use_posx)
+        gtk_window_get_position (GTK_WINDOW (dlg), &options.data.posx, NULL);
+      if (!options.data.use_posy)
+        gtk_window_get_position (GTK_WINDOW (dlg), NULL, &options.data.posy);
+      if (options.data.posx < 0)
+        options.data.posx = sw - ww + options.data.posx;
+      if (options.data.posy < 0)
+        options.data.posy = sh - wh + options.data.posy;
+      gtk_window_move (GTK_WINDOW (dlg), options.data.posx, options.data.posy);
+    }
+}
+
 static GtkWidget *
 create_dialog (void)
 {
@@ -384,9 +366,8 @@ create_dialog (void)
   gtk_container_set_border_width (GTK_CONTAINER (dlg), (guint) options.data.borders);
 
   /* set window size and position */
-  if (!options.data.geometry && !options.data.maximized && !options.data.fullscreen)
+  if (!options.data.maximized && !options.data.fullscreen)
     {
-      gtk_window_set_default_size (GTK_WINDOW (dlg), options.data.width, options.data.height);
       if (options.data.center)
         gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER_ALWAYS);
       else if (options.data.mouse)
@@ -403,11 +384,7 @@ create_dialog (void)
   gtk_window_set_accept_focus (GTK_WINDOW (dlg), options.data.focus);
 
   /* create box */
-#if !GTK_CHECK_VERSION(3,0,0)
-  vbox = gtk_vbox_new (FALSE, 2);
-#else
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-#endif
   gtk_container_add (GTK_CONTAINER (dlg), vbox);
 
   layout = create_layout (dlg);
@@ -429,72 +406,50 @@ create_dialog (void)
         {
           if (strcasecmp (options.data.to_indicator, "top") == 0)
             {
-#if !GTK_CHECK_VERSION(3,0,0)
-              gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (topb), GTK_PROGRESS_LEFT_TO_RIGHT);
-              cbox = gtk_vbox_new (FALSE, 0);
-#else
               gtk_orientable_set_orientation (GTK_ORIENTABLE (topb), GTK_ORIENTATION_HORIZONTAL);
               cbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-#endif
               gtk_box_pack_start (GTK_BOX (cbox), topb, FALSE, FALSE, 2);
               gtk_box_pack_end (GTK_BOX (cbox), layout, TRUE, TRUE, 0);
             }
           else if (strcasecmp (options.data.to_indicator, "bottom") == 0)
             {
-#if !GTK_CHECK_VERSION(3,0,0)
-              gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (topb), GTK_PROGRESS_LEFT_TO_RIGHT);
-              cbox = gtk_vbox_new (FALSE, 0);
-#else
               gtk_orientable_set_orientation (GTK_ORIENTABLE (topb), GTK_ORIENTATION_HORIZONTAL);
               cbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-#endif
               gtk_box_pack_start (GTK_BOX (cbox), layout, TRUE, TRUE, 0);
               gtk_box_pack_end (GTK_BOX (cbox), topb, FALSE, FALSE, 2);
             }
           else if (strcasecmp (options.data.to_indicator, "left") == 0)
             {
-#if !GTK_CHECK_VERSION(3,0,0)
-              gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (topb), GTK_PROGRESS_BOTTOM_TO_TOP);
-              cbox = gtk_hbox_new (FALSE, 0);
-#else
               gtk_orientable_set_orientation (GTK_ORIENTABLE (topb), GTK_ORIENTATION_VERTICAL);
               gtk_progress_bar_set_inverted (GTK_PROGRESS_BAR (topb), TRUE);
               cbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-#endif
               gtk_box_pack_start (GTK_BOX (cbox), topb, FALSE, FALSE, 2);
               gtk_box_pack_end (GTK_BOX (cbox), layout, TRUE, TRUE, 0);
             }
           else if (strcasecmp (options.data.to_indicator, "right") == 0)
             {
-#if !GTK_CHECK_VERSION(3,0,0)
-              gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (topb), GTK_PROGRESS_BOTTOM_TO_TOP);
-              cbox = gtk_hbox_new (FALSE, 0);
-#else
               gtk_orientable_set_orientation (GTK_ORIENTABLE (topb), GTK_ORIENTATION_VERTICAL);
               gtk_progress_bar_set_inverted (GTK_PROGRESS_BAR (topb), TRUE);
               cbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-#endif
               gtk_box_pack_start (GTK_BOX (cbox), layout, TRUE, TRUE, 0);
               gtk_box_pack_end (GTK_BOX (cbox), topb, FALSE, FALSE, 2);
             }
 
-          if (settings.show_remain)
+#ifndef STANDALONE
+          if (g_settings_get_boolean (settings, "show-remain"))
+#else
+          if (SHOW_REMAIN)
+#endif
             {
               gchar *lbl = g_strdup_printf (_("%d sec"), options.data.timeout);
-#if GTK_CHECK_VERSION(3,0,0)
               gtk_progress_bar_set_show_text (GTK_PROGRESS_BAR (topb), TRUE);
-#endif
               gtk_progress_bar_set_text (GTK_PROGRESS_BAR (topb), lbl);
               g_free (lbl);
             }
         }
       else
         {
-#if !GTK_CHECK_VERSION(3,0,0)
-          cbox = gtk_vbox_new (FALSE, 0);
-#else
           cbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-#endif
           gtk_box_pack_start (GTK_BOX (cbox), layout, TRUE, TRUE, 0);
         }
 
@@ -517,11 +472,8 @@ create_dialog (void)
     {
       GtkWidget *btn;
       /* create buttons container */
-#if !GTK_CHECK_VERSION(3,0,0)
-      GtkWidget *bbox = gtk_hbutton_box_new ();
-#else
       GtkWidget *bbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
-#endif
+      gtk_box_set_homogeneous (GTK_BOX (bbox), TRUE);
       gtk_container_set_border_width (GTK_CONTAINER (bbox), 2);
       gtk_box_set_spacing (GTK_BOX (bbox), 5);
       gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), options.data.buttons_layout);
@@ -535,11 +487,10 @@ create_dialog (void)
               YadButton *b = (YadButton *) tmp->data;
 
               btn = gtk_button_new ();
-              gtk_container_add (GTK_CONTAINER (btn), get_label (b->name, 2));
-              gtk_button_set_alignment (GTK_BUTTON (btn), 0.5, 0.5);
+              gtk_container_add (GTK_CONTAINER (btn), get_label (b->name, 2, btn));
               g_object_set_data (G_OBJECT (btn), "resp", GINT_TO_POINTER (b->response));
               g_signal_connect (G_OBJECT (btn), "clicked", G_CALLBACK (btn_cb), b->cmd);
-              gtk_box_pack_start (GTK_BOX (bbox), btn, FALSE, FALSE, 0);
+              gtk_box_pack_start (GTK_BOX (bbox), btn, TRUE, TRUE, 0);
 
               tmp = tmp->next;
             }
@@ -547,46 +498,31 @@ create_dialog (void)
         }
       else
         {
-          if (options.mode == YAD_MODE_PROGRESS || options.mode == YAD_MODE_MULTI_PROGRESS ||
-              options.mode == YAD_MODE_DND || options.mode == YAD_MODE_PICTURE)
+          if (options.mode == YAD_MODE_PROGRESS || options.mode == YAD_MODE_DND || options.mode == YAD_MODE_PICTURE)
             {
               /* add close button */
-              btn = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
+              btn = gtk_button_new ();
+              gtk_container_add (GTK_CONTAINER (btn), get_label ("yad-close", 2, btn));
               g_object_set_data (G_OBJECT (btn), "resp", GINT_TO_POINTER (YAD_RESPONSE_OK));
               g_signal_connect (G_OBJECT (btn), "clicked", G_CALLBACK (btn_cb), NULL);
-              gtk_box_pack_start (GTK_BOX (bbox), btn, FALSE, FALSE, 0);
+              gtk_box_pack_start (GTK_BOX (bbox), btn, TRUE, TRUE, 0);
 
             }
           else
             {
-              if (gtk_alternative_dialog_button_order (NULL))
-                {
-                  /* add ok button */
-                  btn = gtk_button_new_from_stock (GTK_STOCK_OK);
-                  g_object_set_data (G_OBJECT (btn), "resp", GINT_TO_POINTER (YAD_RESPONSE_OK));
-                  g_signal_connect (G_OBJECT (btn), "clicked", G_CALLBACK (btn_cb), NULL);
-                  gtk_box_pack_start (GTK_BOX (bbox), btn, FALSE, FALSE, 0);
+              /* add cancel button */
+              btn = gtk_button_new ();
+              gtk_container_add (GTK_CONTAINER (btn), get_label ("yad-cancel", 2, btn));
+              g_object_set_data (G_OBJECT (btn), "resp", GINT_TO_POINTER (YAD_RESPONSE_CANCEL));
+              g_signal_connect (G_OBJECT (btn), "clicked", G_CALLBACK (btn_cb), NULL);
+              gtk_box_pack_start (GTK_BOX (bbox), btn, TRUE, TRUE, 0);
 
-                  /* add cancel button */
-                  btn = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
-                  g_object_set_data (G_OBJECT (btn), "resp", GINT_TO_POINTER (YAD_RESPONSE_CANCEL));
-                  g_signal_connect (G_OBJECT (btn), "clicked", G_CALLBACK (btn_cb), NULL);
-                  gtk_box_pack_start (GTK_BOX (bbox), btn, FALSE, FALSE, 0);
-                }
-              else
-                {
-                  /* add cancel button */
-                  btn = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
-                  g_object_set_data (G_OBJECT (btn), "resp", GINT_TO_POINTER (YAD_RESPONSE_CANCEL));
-                  g_signal_connect (G_OBJECT (btn), "clicked", G_CALLBACK (btn_cb), NULL);
-                  gtk_box_pack_start (GTK_BOX (bbox), btn, FALSE, FALSE, 0);
-
-                  /*add ok button */
-                  btn = gtk_button_new_from_stock (GTK_STOCK_OK);
-                  g_object_set_data (G_OBJECT (btn), "resp", GINT_TO_POINTER (YAD_RESPONSE_OK));
-                  g_signal_connect (G_OBJECT (btn), "clicked", G_CALLBACK (btn_cb), NULL);
-                  gtk_box_pack_start (GTK_BOX (bbox), btn, FALSE, FALSE, 0);
-                }
+              /*add ok button */
+              btn = gtk_button_new ();
+              gtk_container_add (GTK_CONTAINER (btn), get_label ("yad-ok", 2, btn));
+              g_object_set_data (G_OBJECT (btn), "resp", GINT_TO_POINTER (YAD_RESPONSE_OK));
+              g_signal_connect (G_OBJECT (btn), "clicked", G_CALLBACK (btn_cb), NULL);
+              gtk_box_pack_start (GTK_BOX (bbox), btn, TRUE, TRUE, 0);
             }
         }
       /* add buttons box to main window */
@@ -595,59 +531,35 @@ create_dialog (void)
 
   /* show widgets */
   gtk_widget_show_all (vbox);
+
+  if (options.data.width > 0)
+    gtk_widget_set_size_request (vbox, options.data.width, options.data.height);
+  else
+    {
+      gint mw, nw;
+      gtk_widget_get_preferred_width (vbox, &mw, &nw);
+      gtk_widget_set_size_request (vbox, nw, -1);
+    }
+
   /* parse geometry or move window, if given. must be after showing widget */
   if (!options.data.maximized && !options.data.fullscreen)
     {
-      if (options.data.geometry)
-        {
-          gtk_widget_realize (dlg);
-          gtk_window_parse_geometry (GTK_WINDOW (dlg), options.data.geometry);
-          gtk_widget_show (dlg);
-        }
-      else
-        {
-          gtk_widget_show (dlg);
-          if (options.data.use_posx || options.data.use_posy)
-            {
-              gint ww, wh;
-              gtk_window_get_size (GTK_WINDOW (dlg), &ww, &wh);
-              /* place window to specified coordinates */
-              if (!options.data.use_posx)
-                gtk_window_get_position (GTK_WINDOW (dlg), &options.data.posx, NULL);
-              if (!options.data.use_posy)
-                gtk_window_get_position (GTK_WINDOW (dlg), NULL, &options.data.posy);
-              if (options.data.posx < 0)
-                {
-                  gint sw = gdk_screen_get_width (gdk_screen_get_default ());
-                  options.data.posx = sw - ww + options.data.posx;
-                }
-              if (options.data.posy < 0)
-                {
-                  gint sh = gdk_screen_get_height (gdk_screen_get_default ());
-                  options.data.posy = sh - wh + options.data.posy;
-                }
-              gtk_window_move (GTK_WINDOW (dlg), options.data.posx, options.data.posy);
-            }
-        }
+      parse_geometry ();
+      g_signal_connect (G_OBJECT (dlg), "realize", G_CALLBACK (realize_cb), NULL);
+      gtk_widget_show_all (dlg);
     }
-  else
-    gtk_widget_show (dlg);
-
-  /* set maximized or fixed size after showing widget */
-  if (options.data.maximized)
-    gtk_window_maximize (GTK_WINDOW (dlg));
-  else if (options.data.fullscreen)
-    gtk_window_fullscreen (GTK_WINDOW (dlg));
   else
     {
-      gtk_window_set_resizable (GTK_WINDOW (dlg), !options.data.fixed);
-      if (options.data.fixed)
-        gtk_widget_set_size_request (dlg, options.data.width, options.data.height);
+      gtk_widget_show (dlg);
+      /* set maximized or fixed size after showing widget */
+      if (options.data.maximized)
+        gtk_window_maximize (GTK_WINDOW (dlg));
+      else if (options.data.fullscreen)
+        gtk_window_fullscreen (GTK_WINDOW (dlg));
     }
 
-#ifndef G_OS_WIN32
   /* print xid */
-  if (options.print_xid)
+  if (is_x11 && options.print_xid)
     {
       FILE *xf;
 
@@ -666,12 +578,6 @@ create_dialog (void)
             fflush (xf);
         }
     }
-#endif
-
-#if GTK_CHECK_VERSION(3,0,0)
-  if (css)
-    gtk_style_context_add_provider (gtk_widget_get_style_context (dlg), css, GTK_STYLE_PROVIDER_PRIORITY_USER);
-#endif
 
   return dlg;
 }
@@ -688,6 +594,9 @@ create_plug (void)
       tabs = get_tabs (options.plug, FALSE);
     }
 
+  while (!tabs[0].xid)
+    usleep (1000);
+
   win = gtk_plug_new (0);
   /* set window borders */
   if (options.data.borders == -1)
@@ -698,18 +607,12 @@ create_plug (void)
   if (box)
     gtk_container_add (GTK_CONTAINER (win), box);
 
-#if GTK_CHECK_VERSION(3,0,0)
-  if (css)
-    gtk_style_context_add_provider (gtk_widget_get_style_context (win), css, GTK_STYLE_PROVIDER_PRIORITY_USER);
-#endif
-
   gtk_widget_show_all (win);
 
   /* add plug data */
+  /* notebook/paned will count non-zero xids */
   tabs[options.tabnum].pid = getpid ();
   tabs[options.tabnum].xid = gtk_plug_get_id (GTK_PLUG (win));
-  /* FIXME: may be a race here */
-  tabs[0].xid++;
   shmdt (tabs);
 }
 
@@ -718,6 +621,9 @@ yad_print_result (void)
 {
   switch (options.mode)
     {
+    case YAD_MODE_APP:
+      app_print_result ();
+      break;
     case YAD_MODE_CALENDAR:
       calendar_print_result ();
       break;
@@ -771,13 +677,15 @@ main (gint argc, gchar ** argv)
   textdomain (GETTEXT_PACKAGE);
 #endif
 
-#if !GLIB_CHECK_VERSION(2,36,0)
-  g_type_init ();
-#endif
-  read_settings ();
-
   gtk_init (&argc, &argv);
   g_set_application_name ("YAD");
+
+#ifndef STANDALONE
+  settings = g_settings_new ("yad.settings");
+#endif
+
+  yad_icon_theme = gtk_icon_theme_get_default ();
+
   yad_options_init ();
 
   ctx = yad_create_context ();
@@ -817,31 +725,29 @@ main (gint argc, gchar ** argv)
     }
   yad_set_mode ();
 
+  /* check for current GDK backend */
+#ifdef GDK_WINDOWING_X11
+  if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+    is_x11 = TRUE;
+#endif
+
   /* parse custom gtkrc */
   if (options.gtkrc_file)
     {
-#if !GTK_CHECK_VERSION(3,0,0)
-      gtk_rc_parse (options.gtkrc_file);
-#else
-      css = gtk_css_provider_new ();
+      GtkCssProvider *css = gtk_css_provider_new ();
       gtk_css_provider_load_from_path (css, options.gtkrc_file, NULL);
-#endif
+      gtk_style_context_add_provider_for_screen (gdk_screen_get_default (), GTK_STYLE_PROVIDER (css),
+                                                 GTK_STYLE_PROVIDER_PRIORITY_USER);
+      g_object_unref (css);
     }
 
   /* set default icons and icon theme */
   if (options.data.icon_theme)
-    {
-      settings.icon_theme = gtk_icon_theme_new ();
-      gtk_icon_theme_set_custom_theme (settings.icon_theme, options.data.icon_theme);
-    }
-  else
-    settings.icon_theme = gtk_icon_theme_get_default ();
+    gtk_icon_theme_set_custom_theme (yad_icon_theme, options.data.icon_theme);
   gtk_icon_size_lookup (GTK_ICON_SIZE_DIALOG, &w, &h);
-  settings.big_fallback_image =
-    gtk_icon_theme_load_icon (settings.icon_theme, "yad", MIN (w, h), GTK_ICON_LOOKUP_GENERIC_FALLBACK, NULL);
+  big_fallback_image = gtk_icon_theme_load_icon (yad_icon_theme, "yad", MIN (w, h), GTK_ICON_LOOKUP_GENERIC_FALLBACK, NULL);
   gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
-  settings.small_fallback_image =
-    gtk_icon_theme_load_icon (settings.icon_theme, "yad", MIN (w, h), GTK_ICON_LOOKUP_GENERIC_FALLBACK, NULL);
+  small_fallback_image = gtk_icon_theme_load_icon (yad_icon_theme, "yad", MIN (w, h), GTK_ICON_LOOKUP_GENERIC_FALLBACK, NULL);
 
   /* correct separators */
   str = g_strcompress (options.common_data.separator);
@@ -892,6 +798,13 @@ main (gint argc, gchar ** argv)
   signal (SIGUSR2, sa_usr2);
 #endif
 
+  if (!is_x11 && options.plug != -1)
+    {
+      options.plug = -1;
+      if (options.debug)
+        g_printerr (_("WARNING: --plug mode not supported outside X11\n"));
+    }
+
   /* plug mode */
   if (options.plug != -1)
     {
@@ -901,6 +814,19 @@ main (gint argc, gchar ** argv)
       return ret;
     }
 
+  if (!is_x11)
+    {
+      if (options.mode == YAD_MODE_NOTEBOOK || options.mode == YAD_MODE_PANED
+#ifdef HAVE_TRAY
+          || options.mode == YAD_MODE_NOTIFICATION
+#endif
+         )
+        {
+          g_printerr (_("WARNING: This mode not supported outside X11\n"));
+          return 1;
+        }
+    }
+
   switch (options.mode)
     {
     case YAD_MODE_ABOUT:
@@ -923,9 +849,11 @@ main (gint argc, gchar ** argv)
       break;
 #endif
 
+#ifdef HAVE_TRAY
     case YAD_MODE_NOTIFICATION:
       ret = yad_notification_run ();
       break;
+#endif
 
     case YAD_MODE_PRINT:
       ret = yad_print_run ();
@@ -934,11 +862,12 @@ main (gint argc, gchar ** argv)
     default:
       dialog = create_dialog ();
 
-#ifndef G_OS_WIN32
-      /* add YAD_XID variable */
-      str = g_strdup_printf ("0x%X", (guint) GDK_WINDOW_XID (gtk_widget_get_window (dialog)));
-      g_setenv ("YAD_XID", str, TRUE);
-#endif
+      if (is_x11)
+        {
+          /* add YAD_XID variable */
+          str = g_strdup_printf ("0x%X", (guint) GDK_WINDOW_XID (gtk_widget_get_window (dialog)));
+          g_setenv ("YAD_XID", str, TRUE);
+        }
 
       /* make some specific init actions */
       if (options.mode == YAD_MODE_NOTEBOOK)
@@ -951,6 +880,9 @@ main (gint argc, gchar ** argv)
             picture_fit_to_window ();
         }
 
+      if (text && options.data.selectable_labels)
+        gtk_label_select_region (GTK_LABEL (text), 0, 0);
+
       /* run main loop */
       gtk_main ();
 
@@ -977,8 +909,7 @@ main (gint argc, gchar ** argv)
       /* autokill option for progress dialog */
       if (!options.kill_parent)
         {
-          if ((options.mode == YAD_MODE_PROGRESS || options.mode == YAD_MODE_MULTI_PROGRESS) &&
-              options.progress_data.autokill && ret != YAD_RESPONSE_OK)
+          if (options.mode == YAD_MODE_PROGRESS && options.progress_data.autokill && ret != YAD_RESPONSE_OK)
             kill (getppid (), SIGHUP);
         }
 #endif
