@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <sys/types.h>
@@ -55,25 +55,18 @@ notebook_create_widget (GtkWidget * dlg)
   /* add tabs */
   for (tab = options.notebook_data.tabs; tab; tab = tab->next)
     {
-      GtkWidget *a, *s;
-
-      a = gtk_alignment_new (0.5, 0.5, 1, 1);
-      gtk_alignment_set_padding (GTK_ALIGNMENT (a),
-                                 options.notebook_data.borders, options.notebook_data.borders,
-                                 options.notebook_data.borders, options.notebook_data.borders);
+      GtkWidget *s;
 
       s = gtk_socket_new ();
-      gtk_container_add (GTK_CONTAINER (a), s);
-      g_object_set_data (G_OBJECT (a), "socket", s);
+      gtk_widget_set_margin_start (s, options.notebook_data.borders);
+      gtk_widget_set_margin_end (s, options.notebook_data.borders);
+      gtk_widget_set_margin_top (s, options.notebook_data.borders);
+      gtk_widget_set_margin_bottom (s, options.notebook_data.borders);
 
-      gtk_notebook_append_page (GTK_NOTEBOOK (w), a, get_label ((gchar *) tab->data, 0));
+      gtk_notebook_append_page (GTK_NOTEBOOK (w), s, get_label ((gchar *) tab->data, 0, s));
+      gtk_container_child_set (GTK_CONTAINER (w), s, "tab-expand", options.notebook_data.expand, NULL);
     }
 
-  /* set active tab */
-  if (options.notebook_data.active <= 0)
-    options.notebook_data.active = 1;
-  gtk_notebook_set_current_page (GTK_NOTEBOOK (w), options.notebook_data.active - 1);
-
   return w;
 }
 
@@ -81,22 +74,36 @@ void
 notebook_swallow_childs (void)
 {
   guint i, n_tabs;
+  gboolean all_registered;
 
   n_tabs = g_slist_length (options.notebook_data.tabs);
 
-  /* wait until all children are register */
-  while (tabs[0].xid != n_tabs)
-    usleep (1000);
+  /* wait until all children are registered */
+  do
+    {
+      all_registered = TRUE;
+      for (i = 1; i <= n_tabs; i++)
+        if (!tabs[i].xid)
+          {
+            all_registered = FALSE;
+            break;
+          }
+      if (!all_registered)
+        usleep (1000);
+    }
+  while (!all_registered);
 
   for (i = 1; i <= n_tabs; i++)
     {
-      GtkWidget *s =
-        GTK_WIDGET (g_object_get_data
-                    (G_OBJECT (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i - 1)), "socket"));
+      GtkWidget *s = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i - 1);
 
       if (tabs[i].pid != -1)
         gtk_socket_add_id (GTK_SOCKET (s), tabs[i].xid);
     }
+
+  /* set active tab */
+  if (options.notebook_data.active > 0)
+    gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), options.notebook_data.active - 1);
 }
 
 void
@@ -117,17 +124,19 @@ notebook_close_childs (void)
 {
   guint i, n_tabs;
   struct shmid_ds buf;
-  gboolean is_running = TRUE;
+  gboolean is_running;
 
   n_tabs = g_slist_length (options.notebook_data.tabs);
   for (i = 1; i <= n_tabs; i++)
     {
       if (tabs[i].pid != -1)
         kill (tabs[i].pid, SIGUSR2);
+      else
+        break;
     }
 
   /* wait for stop subprocesses */
-  while (is_running)
+  do
     {
       is_running = FALSE;
       for (i = 1; i <= n_tabs; i++)
@@ -138,8 +147,10 @@ notebook_close_childs (void)
               break;
             }
         }
-      usleep (1000);
+      if (is_running)
+        usleep (1000);
     }
+  while (is_running);
 
   /* cleanup shared memory */
   shmctl (tabs[0].pid, IPC_RMID, &buf);
