@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <sys/stat.h>
@@ -22,7 +22,6 @@
 #include <time.h>
 #include <string.h>
 #include <unistd.h>
-#include <errno.h>
 
 #include "yad.h"
 
@@ -41,15 +40,19 @@ static GSList *menu_data = NULL;
 
 static gint exit_code;
 
+static void popup_menu_cb (GtkStatusIcon *, guint, guint, gpointer);
+
 static void
-free_menu_data (gpointer data, gpointer udata)
+free_menu_data (gpointer data)
 {
   MenuData *m = (MenuData *) data;
 
-  g_free (m->name);
-  g_free (m->action);
-  g_free (m->icon);
-  g_free (m);
+  if (m) {
+    g_free (m->name);
+    g_free (m->action);
+    g_free (m->icon);
+    g_free (m);
+  }
 }
 
 static void
@@ -60,8 +63,7 @@ parse_menu_str (gchar * str)
 
   if (menu_data)
     {
-      g_slist_foreach (menu_data, free_menu_data, NULL);
-      g_slist_free (menu_data);
+      g_slist_free_full (menu_data, free_menu_data);
       menu_data = NULL;
     }
 
@@ -74,7 +76,14 @@ parse_menu_str (gchar * str)
 
       if (s[0])
         {
-          mdata->name = g_strdup (s[0]);
+          YadStock sit;
+          if (stock_lookup (s[0], &sit))
+            {
+              mdata->name = g_strdup (sit.label);
+              mdata->icon = g_strdup (sit.icon);
+            }
+          else
+              mdata->name = g_strdup (s[0]);
           if (s[1])
             {
               mdata->action = g_strdup (s[1]);
@@ -105,14 +114,17 @@ set_icon (void)
 
   if (icon == NULL)
     {
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
       gtk_status_icon_set_from_icon_name (status_icon, "yad");
+      G_GNUC_END_IGNORE_DEPRECATIONS;
       return;
     }
 
   if (g_file_test (icon, G_FILE_TEST_EXISTS))
     {
-      pixbuf = gdk_pixbuf_new_from_file_at_scale (icon, options.notification_data.icon_size, 
-                                                  options.notification_data.icon_size, TRUE, &err);
+      gint isize = (options.common_data.icon_size > 0) ? options.common_data.icon_size : 16;
+
+      pixbuf = gdk_pixbuf_new_from_file_at_scale (icon, isize, isize, TRUE, &err);
       if (err)
         {
           g_printerr (_("Could not load notification icon '%s': %s\n"), icon, err->message);
@@ -120,14 +132,24 @@ set_icon (void)
         }
       if (pixbuf)
         {
+          G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
           gtk_status_icon_set_from_pixbuf (status_icon, pixbuf);
+          G_GNUC_END_IGNORE_DEPRECATIONS;
           g_object_unref (pixbuf);
         }
       else
-        gtk_status_icon_set_from_icon_name (status_icon, "yad");
+        {
+          G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
+          gtk_status_icon_set_from_icon_name (status_icon, "yad");
+          G_GNUC_END_IGNORE_DEPRECATIONS;
+        }
     }
   else
-    gtk_status_icon_set_from_icon_name (status_icon, icon);
+    {
+      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
+      gtk_status_icon_set_from_icon_name (status_icon, icon);
+      G_GNUC_END_IGNORE_DEPRECATIONS;
+    }
 }
 
 static gboolean
@@ -139,7 +161,12 @@ activate_cb (GtkWidget * widget, YadData * data)
       gtk_main_quit ();
     }
   else if (action)
-    g_spawn_command_line_async (action, NULL);
+    {
+      if (g_ascii_strcasecmp (action, "menu") == 0)
+        popup_menu_cb (GTK_STATUS_ICON (widget), 1, GDK_CURRENT_TIME, data);
+      else
+        run_command_async (action);
+    }
 
   return TRUE;
 }
@@ -166,18 +193,18 @@ popup_menu_item_activate_cb (GtkWidget * w, gpointer data)
 
   if (cmd)
     {
-      if (g_ascii_strcasecmp (cmd, "quit") == 0)
+      if (g_ascii_strcasecmp (g_strstrip (cmd), "quit") == 0)
         {
           exit_code = YAD_RESPONSE_OK;
           gtk_main_quit ();
         }
       else
-        g_spawn_command_line_async (cmd, NULL);
+        run_command_async (cmd);
     }
 }
 
 static void
-popup_menu_cb (GtkStatusIcon * icon, guint button, guint activate_time, gpointer data)
+popup_menu_cb (GtkStatusIcon *icon, guint button, guint activate_time, gpointer data)
 {
   GtkWidget *menu;
   GtkWidget *item;
@@ -187,40 +214,47 @@ popup_menu_cb (GtkStatusIcon * icon, guint button, guint activate_time, gpointer
     return;
 
   menu = gtk_menu_new ();
+  gtk_menu_set_reserve_toggle_size (GTK_MENU (menu), FALSE);
+
   for (m = menu_data; m; m = m->next)
     {
       MenuData *d = (MenuData *) m->data;
 
       if (d->name)
         {
+          GtkWidget *b, *i, *l;
+
+          item = gtk_menu_item_new ();
+          b = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
+
           if (d->icon)
             {
-              GdkPixbuf *pb = get_pixbuf (d->icon, YAD_SMALL_ICON);
-              item = gtk_image_menu_item_new_with_mnemonic (d->name);
+              GdkPixbuf *pb = get_pixbuf (d->icon, YAD_SMALL_ICON, TRUE);
               if (pb)
                 {
-                  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), gtk_image_new_from_pixbuf (pb));
+                  i = gtk_image_new_from_pixbuf (pb);
+                  gtk_container_add (GTK_CONTAINER (b), i);
                   g_object_unref (pb);
                 }
             }
-          else
-            {
-              GtkStockItem it;
-              if (gtk_stock_lookup (d->name, &it))
-                item = gtk_image_menu_item_new_from_stock (d->name, NULL);
-              else
-                item = gtk_menu_item_new_with_mnemonic (d->name);
-            }
+          l = gtk_label_new_with_mnemonic (d->name);
+          gtk_label_set_xalign (GTK_LABEL (l), 0.0);
+          gtk_label_set_mnemonic_widget (GTK_LABEL (l), item);
+          gtk_box_pack_end (GTK_BOX (b), l, TRUE, TRUE, 0);
+
+          gtk_container_add (GTK_CONTAINER (item), b);
+
           g_signal_connect (GTK_MENU_ITEM (item), "activate",
                             G_CALLBACK (popup_menu_item_activate_cb), (gpointer) d->action);
         }
       else
         item = gtk_separator_menu_item_new ();
 
-      gtk_widget_show (item);
+      gtk_widget_show_all (item);
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
     }
-  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, gtk_status_icon_position_menu, icon, button, activate_time);
+
+  gtk_menu_popup_at_pointer (GTK_MENU (menu), NULL);
 }
 
 static gboolean
@@ -232,7 +266,8 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
       GError *err = NULL;
 
       string = g_string_new (NULL);
-      while (channel->is_readable == FALSE);
+      while (channel->is_readable == FALSE)
+        usleep (100);
 
       do
         {
@@ -278,18 +313,22 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
               g_free (icon);
               icon = g_strdup (value);
 
+              G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
               if (gtk_status_icon_get_visible (status_icon) && gtk_status_icon_is_embedded (status_icon))
                 set_icon ();
+              G_GNUC_END_IGNORE_DEPRECATIONS;
             }
           else if (!g_ascii_strcasecmp (command, "tooltip"))
             {
               if (g_utf8_validate (value, -1, NULL))
                 {
                   gchar *message = g_strcompress (value);
+                  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
                   if (!options.data.no_markup)
                     gtk_status_icon_set_tooltip_markup (status_icon, message);
                   else
                     gtk_status_icon_set_tooltip_text (status_icon, message);
+                  G_GNUC_END_IGNORE_DEPRECATIONS;
                   g_free (message);
                 }
               else
@@ -297,28 +336,12 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
             }
           else if (!g_ascii_strcasecmp (command, "visible"))
             {
-#if !GTK_CHECK_VERSION(2,22,0)
-              if (!g_ascii_strcasecmp (value, "blink"))
-                {
-                  gboolean state = gtk_status_icon_get_blinking (status_icon);
-                  gtk_status_icon_set_blinking (status_icon, !state);
-                }
-              else
-#endif
+              G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
               if (!g_ascii_strcasecmp (value, "false"))
-                {
-                  gtk_status_icon_set_visible (status_icon, FALSE);
-#if !GTK_CHECK_VERSION(2,22,0)
-                  gtk_status_icon_set_blinking (status_icon, FALSE);
-#endif
-                }
+                gtk_status_icon_set_visible (status_icon, FALSE);
               else
-                {
-                  gtk_status_icon_set_visible (status_icon, TRUE);
-#if !GTK_CHECK_VERSION(2,22,0)
-                  gtk_status_icon_set_blinking (status_icon, FALSE);
-#endif
-                }
+                gtk_status_icon_set_visible (status_icon, TRUE);
+              G_GNUC_END_IGNORE_DEPRECATIONS;
             }
           else if (!g_ascii_strcasecmp (command, "action"))
             {
@@ -361,8 +384,11 @@ yad_notification_run ()
 {
   GIOChannel *channel = NULL;
 
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
   status_icon = gtk_status_icon_new ();
+  G_GNUC_END_IGNORE_DEPRECATIONS;
 
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
   if (options.data.dialog_text)
     {
       if (!options.data.no_markup)
@@ -372,6 +398,7 @@ yad_notification_run ()
     }
   else
     gtk_status_icon_set_tooltip_text (status_icon, _("Yad notification"));
+  G_GNUC_END_IGNORE_DEPRECATIONS;
 
   if (options.data.dialog_image)
     icon = g_strdup (options.data.dialog_image);
@@ -402,7 +429,9 @@ yad_notification_run ()
     }
 
   /* Show icon and wait */
+  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
   gtk_status_icon_set_visible (status_icon, !options.notification_data.hidden);
+  G_GNUC_END_IGNORE_DEPRECATIONS;
 
   if (options.data.timeout > 0)
     g_timeout_add_seconds (options.data.timeout, (GSourceFunc) timeout_cb, NULL);
