@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include <sys/types.h>
@@ -46,10 +46,7 @@ paned_create_widget (GtkWidget * dlg)
     exit (-1);
 
   /* create widget */
-  if (options.paned_data.orient == GTK_ORIENTATION_HORIZONTAL)
-    paned = w = gtk_hpaned_new ();
-  else
-    paned = w = gtk_vpaned_new ();
+  paned = w = gtk_paned_new (options.paned_data.orient);
   gtk_widget_set_name (w, "yad-paned-widget");
 
   gtk_paned_set_position (GTK_PANED (w), options.paned_data.splitter);
@@ -73,8 +70,8 @@ paned_swallow_childs (void)
   s1 = GTK_WIDGET (g_object_get_data (G_OBJECT (paned), "s1"));
   s2 = GTK_WIDGET (g_object_get_data (G_OBJECT (paned), "s2"));
 
-  /* wait until all children are register */
-  while (tabs[0].xid != 2)
+  /* wait until all children are registered */
+  while (!tabs[1].xid || !tabs[2].xid)
     usleep (1000);
 
   if (tabs[1].pid != -1)
@@ -108,7 +105,7 @@ paned_close_childs (void)
   while (is_running)
     {
       is_running = FALSE;
-      for (i = 1; i <= 3; i++)
+      for (i = 1; i < 3; i++)
         {
           if (tabs[i].pid != -1 && kill (tabs[i].pid, 0) == 0)
             {
@@ -116,7 +113,8 @@ paned_close_childs (void)
               break;
             }
         }
-      usleep (1000);
+      if (is_running)
+        usleep (1000);
     }
 
   /* cleanup shared memory */
