@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include "yad.h"
@@ -61,7 +61,7 @@ load_picture (gchar *filename)
       loaded = TRUE;
     }
   else
-    gtk_image_set_from_stock (GTK_IMAGE (picture), "gtk-missing-image", GTK_ICON_SIZE_DIALOG);
+    gtk_image_set_from_icon_name (GTK_IMAGE (picture), "image-missing", GTK_ICON_SIZE_DIALOG);
 }
 
 void
@@ -84,7 +84,11 @@ picture_fit_to_window ()
     {
       GdkPixbuf *pb = gdk_pixbuf_scale_simple (g_object_ref (orig_pb), width * factor, height * factor, GDK_INTERP_HYPER);
       if (pb)
-        gtk_image_set_from_pixbuf (GTK_IMAGE (picture), pb);
+        {
+          GdkPixbuf *old_pb = gtk_image_get_pixbuf (GTK_IMAGE (picture));
+          gtk_image_set_from_pixbuf (GTK_IMAGE (picture), pb);
+          g_object_unref (old_pb);
+        }
     }
 }
 
@@ -110,6 +114,7 @@ change_size_cb (GtkWidget *w, gint type)
       break;
     case SIZE_ORIG:
       gtk_image_set_from_pixbuf (GTK_IMAGE (picture), orig_pb);
+      g_object_unref (pb);
       break;
     case SIZE_INC:
       new_pb = gdk_pixbuf_scale_simple (pb, width + options.picture_data.inc,
@@ -173,31 +178,24 @@ create_popup_menu ()
   GtkWidget *mi;
 
   popup_menu = gtk_menu_new ();
+  gtk_menu_set_reserve_toggle_size (GTK_MENU (popup_menu), FALSE);
 
-  mi = gtk_image_menu_item_new_with_label (_("Fit to window"));
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi),
-                                 gtk_image_new_from_icon_name ("gtk-zoom-fit", GTK_ICON_SIZE_MENU));
+  mi = gtk_menu_item_new_with_label (_("Fit to window"));
   gtk_widget_show (mi);
   gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), mi);
   g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (change_size_cb), GINT_TO_POINTER (SIZE_FIT));
 
-  mi = gtk_image_menu_item_new_with_label (_("Original size"));
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi),
-                                 gtk_image_new_from_icon_name ("gtk-zoom-100", GTK_ICON_SIZE_MENU));
+  mi = gtk_menu_item_new_with_label (_("Original size"));
   gtk_widget_show (mi);
   gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), mi);
   g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (change_size_cb), GINT_TO_POINTER (SIZE_ORIG));
 
-  mi = gtk_image_menu_item_new_with_label (_("Increase size"));
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi),
-                                 gtk_image_new_from_icon_name ("gtk-zoom-in", GTK_ICON_SIZE_MENU));
+  mi = gtk_menu_item_new_with_label (_("Increase size"));
   gtk_widget_show (mi);
   gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), mi);
   g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (change_size_cb), GINT_TO_POINTER (SIZE_INC));
 
-  mi = gtk_image_menu_item_new_with_label (_("Decrease size"));
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi),
-                                 gtk_image_new_from_icon_name ("gtk-zoom-out", GTK_ICON_SIZE_MENU));
+  mi = gtk_menu_item_new_with_label (_("Decrease size"));
   gtk_widget_show (mi);
   gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), mi);
   g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (change_size_cb), GINT_TO_POINTER (SIZE_DEC));
@@ -206,30 +204,22 @@ create_popup_menu ()
   gtk_widget_show (mi);
   gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), mi);
 
-  mi = gtk_image_menu_item_new_with_label (_("Rotate left"));
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi),
-                                 gtk_image_new_from_icon_name ("object-rotate-left", GTK_ICON_SIZE_MENU));
+  mi = gtk_menu_item_new_with_label (_("Rotate left"));
   gtk_widget_show (mi);
   gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), mi);
   g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (rotate_cb), GINT_TO_POINTER (ROTATE_LEFT));
 
-  mi = gtk_image_menu_item_new_with_label (_("Rotate right"));
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi),
-                                 gtk_image_new_from_icon_name ("object-rotate-right", GTK_ICON_SIZE_MENU));
+  mi = gtk_menu_item_new_with_label (_("Rotate right"));
   gtk_widget_show (mi);
   gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), mi);
   g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (rotate_cb), GINT_TO_POINTER (ROTATE_RIGHT));
 
-  mi = gtk_image_menu_item_new_with_label (_("Flip vertical"));
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi),
-                                 gtk_image_new_from_icon_name ("object-flip-vertical", GTK_ICON_SIZE_MENU));
+  mi = gtk_menu_item_new_with_label (_("Flip vertical"));
   gtk_widget_show (mi);
   gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), mi);
   g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (rotate_cb), GINT_TO_POINTER (ROTATE_FLIP_VERT));
 
-  mi = gtk_image_menu_item_new_with_label (_("Flip horizontal"));
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi),
-                                 gtk_image_new_from_icon_name ("object-flip-horizontal", GTK_ICON_SIZE_MENU));
+  mi = gtk_menu_item_new_with_label (_("Flip horizontal"));
   gtk_widget_show (mi);
   gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), mi);
   g_signal_connect (G_OBJECT (mi), "activate", G_CALLBACK (rotate_cb), GINT_TO_POINTER (ROTATE_FLIP_HOR));
@@ -240,7 +230,7 @@ button_handler (GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
   if (ev->button == 3)
     {
-      gtk_menu_popup (GTK_MENU (popup_menu), NULL, NULL, NULL, NULL, ev->button, ev->time);
+      gtk_menu_popup_at_pointer (GTK_MENU (popup_menu), NULL);
       return TRUE;
     }
 
@@ -256,7 +246,7 @@ key_handler (GtkWidget *w, GdkEventKey *ev, gpointer data)
 GtkWidget *
 picture_create_widget (GtkWidget * dlg)
 {
-  GtkWidget *sw;
+  GtkWidget *sw, *ev;
 
   sw = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE);
@@ -266,21 +256,24 @@ picture_create_widget (GtkWidget * dlg)
                                gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (sw)));
   gtk_container_add (GTK_CONTAINER (sw), viewport);
 
+  ev = gtk_event_box_new ();
+  gtk_container_add (GTK_CONTAINER (viewport), ev);
+
   picture = gtk_image_new ();
-  gtk_container_add (GTK_CONTAINER (viewport), picture);
+  gtk_container_add (GTK_CONTAINER (ev), picture);
 
   /* load picture */
   if (options.common_data.uri &&
       g_file_test (options.common_data.uri, G_FILE_TEST_EXISTS))
     load_picture (options.common_data.uri);
   else
-    gtk_image_set_from_stock (GTK_IMAGE (picture), "gtk-missing-image", GTK_ICON_SIZE_DIALOG);
+    gtk_image_set_from_icon_name (GTK_IMAGE (picture), "image-missing", GTK_ICON_SIZE_DIALOG);
 
   if (loaded && !animated)
     {
       create_popup_menu ();
-      g_signal_connect (G_OBJECT (viewport), "button-press-event", G_CALLBACK (button_handler), NULL);
-      g_signal_connect (G_OBJECT (viewport), "key-press-event", G_CALLBACK (key_handler), NULL);
+      g_signal_connect (G_OBJECT (ev), "button-press-event", G_CALLBACK (button_handler), NULL);
+      g_signal_connect (G_OBJECT (ev), "key-press-event", G_CALLBACK (key_handler), NULL);
     }
 
   return sw;
