@@ -14,16 +14,18 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #include "yad.h"
+#include <math.h>
 
 enum {
   PLUS_BTN = 0,
   MINUS_BTN,
 };
 
+static GtkAdjustment *adj;
 static GtkWidget *scale;
 static GtkWidget *plus_btn = NULL;
 static GtkWidget *minus_btn = NULL;
@@ -31,6 +33,14 @@ static GtkWidget *minus_btn = NULL;
 static void
 value_changed_cb (GtkWidget * w, gpointer data)
 {
+  if (options.scale_data.enforce_step)
+  {
+    /* Make sure the new value corresponds to a step in the range. */
+    gdouble value = gtk_adjustment_get_value ((GtkAdjustment *)adj);
+    gdouble new_value = (gdouble) round(value/options.scale_data.step) * options.scale_data.step;
+    gtk_adjustment_set_value ((GtkAdjustment *)adj, new_value);
+  }
+
   if (options.scale_data.print_partial)
     g_print ("%.0f\n", gtk_range_get_value (GTK_RANGE (scale)));
 
@@ -72,7 +82,6 @@ GtkWidget *
 scale_create_widget (GtkWidget * dlg)
 {
   GtkWidget *w;
-  GtkAdjustment *adj;
   gint page;
 
   if (options.scale_data.min_value >= options.scale_data.max_value)
@@ -108,20 +117,12 @@ scale_create_widget (GtkWidget * dlg)
                                               0.0);
   if (options.common_data.vertical)
     {
-#if GTK_CHECK_VERSION(3,0,0)
       scale = gtk_scale_new (GTK_ORIENTATION_VERTICAL, GTK_ADJUSTMENT (adj));
-#else
-      scale = gtk_vscale_new (GTK_ADJUSTMENT (adj));
-#endif
       gtk_range_set_inverted (GTK_RANGE (scale), !options.scale_data.invert);
     }
   else
     {
-#if GTK_CHECK_VERSION(3,0,0)
       scale = gtk_scale_new (GTK_ORIENTATION_HORIZONTAL, GTK_ADJUSTMENT (adj));
-#else
-      scale = gtk_hscale_new (GTK_ADJUSTMENT (adj));
-#endif
       gtk_range_set_inverted (GTK_RANGE (scale), options.scale_data.invert);
     }
   gtk_widget_set_name (scale, "yad-scale-widget");
@@ -146,21 +147,9 @@ scale_create_widget (GtkWidget * dlg)
 
   /* create container */
   if (options.common_data.vertical)
-    {
-#if GTK_CHECK_VERSION(3,0,0)
-      w = gtk_box_new (GTK_ORIENTATION_VERTICAL, 1);
-#else
-      w = gtk_vbox_new (FALSE, 1);
-#endif
-    }
+    w = gtk_box_new (GTK_ORIENTATION_VERTICAL, 1);
   else
-    {
-#if GTK_CHECK_VERSION(3,0,0)
-      w = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 1);
-#else
-      w = gtk_hbox_new (FALSE, 1);
-#endif
-    }
+    w = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 1);
 
   /* create buttons */
   if (options.scale_data.buttons)
