@@ -14,11 +14,9 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
-#include <errno.h>
-
 #include <pango/pango.h>
 
 #include "yad.h"
@@ -88,11 +86,7 @@ do_search (GtkWidget * e, GtkWidget * w)
 static gboolean
 search_key_cb (GtkWidget * w, GdkEventKey * key, GtkWidget * win)
 {
-#if GTK_CHECK_VERSION(2,24,0)
   if (key->keyval == GDK_KEY_Escape)
-#else
-  if (key->keyval == GDK_Escape)
-#endif
     {
       gtk_widget_destroy (win);
       return TRUE;
@@ -109,7 +103,7 @@ search_changed (GtkWidget * w, gpointer d)
 static void
 show_search ()
 {
-  GtkWidget *w, *f, *a, *e;
+  GtkWidget *w, *f, *e;
   GdkEvent *fev;
 
   w = gtk_window_new (GTK_WINDOW_POPUP);
@@ -123,16 +117,13 @@ show_search ()
 
   f = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (f), GTK_SHADOW_ETCHED_IN);
+  gtk_container_set_border_width (GTK_CONTAINER (f), 1);
   gtk_container_add (GTK_CONTAINER (w), f);
 
-  a = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
-  gtk_alignment_set_padding (GTK_ALIGNMENT (a), 2, 2, 2, 2);
-  gtk_container_add (GTK_CONTAINER (f), a);
-
   e = gtk_entry_new ();
   if (pattern)
     gtk_entry_set_text (GTK_ENTRY (e), pattern);
-  gtk_container_add (GTK_CONTAINER (a), e);
+  gtk_container_add (GTK_CONTAINER (f), e);
 
   g_signal_connect (G_OBJECT (e), "activate", G_CALLBACK (do_search), w);
   g_signal_connect (G_OBJECT (e), "changed", G_CALLBACK (search_changed), NULL);
@@ -147,26 +138,14 @@ show_search ()
   fev->focus_change.window = gtk_widget_get_window (e);
   if (fev->focus_change.window != NULL)
     g_object_ref (fev->focus_change.window);
-#if GTK_CHECK_VERSION(2,22,0)
   gtk_widget_send_focus_change (e, fev);
-#else
-  g_object_ref (e);
-  GTK_OBJECT_FLAGS (e) |= GTK_HAS_FOCUS;
-  gtk_widget_event (e, fev);
-  g_object_notify (G_OBJECT (e), "has-focus");
-  g_object_unref (e);
-#endif
   gdk_event_free (fev);
 }
 
 static gboolean
 key_press_cb (GtkWidget * w, GdkEventKey * key, gpointer data)
 {
-#if GTK_CHECK_VERSION(2,24,0)
   if ((key->state & GDK_CONTROL_MASK) && (key->keyval == GDK_KEY_S || key->keyval == GDK_KEY_s))
-#else
-  if ((key->state & GDK_CONTROL_MASK) && (key->keyval == GDK_S || key->keyval == GDK_s))
-#endif
     {
       show_search ();
       return TRUE;
@@ -180,7 +159,7 @@ tag_event_cb (GtkTextTag * tag, GObject * obj, GdkEvent * ev, GtkTextIter * iter
 {
   GtkTextIter start = *iter;
   GtkTextIter end = *iter;
-  gchar *url, *cmdline;
+  gchar *url;
 
   if (ev->type == GDK_BUTTON_PRESS)
     {
@@ -192,12 +171,9 @@ tag_event_cb (GtkTextTag * tag, GObject * obj, GdkEvent * ev, GtkTextIter * iter
           gtk_text_iter_forward_to_tag_toggle (&end, tag);
 
           url = gtk_text_iter_get_text (&start, &end);
-          cmdline = g_strdup_printf (settings.open_cmd, url);
+          open_uri (url);
           g_free (url);
 
-          g_spawn_command_line_async (cmdline, NULL);
-
-          g_free (cmdline);
           return TRUE;
         }
     }
@@ -245,8 +221,6 @@ motion_cb (GtkWidget * w, GdkEventMotion * ev, gpointer d)
   if (tags)
     g_slist_free (tags);
 
-  gdk_window_get_pointer (gtk_widget_get_window (w), NULL, NULL, NULL);
-
   return FALSE;
 }
 
@@ -296,7 +270,8 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
       gint status;
 
       string = g_string_new (NULL);
-      while (channel->is_readable != TRUE);
+      while (channel->is_readable != TRUE)
+        usleep (100);
 
       do
         {
@@ -338,11 +313,19 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
             {
               gchar *utftext =
                 g_convert_with_fallback (string->str, string->len, "UTF-8", "ISO-8859-1", NULL, NULL, NULL, NULL);
-              gtk_text_buffer_insert (GTK_TEXT_BUFFER (text_buffer), &end, utftext, -1);
+              if (options.text_data.formatted && !options.common_data.editable)
+                gtk_text_buffer_insert_markup (GTK_TEXT_BUFFER (text_buffer), &end, utftext, -1);
+              else
+                gtk_text_buffer_insert (GTK_TEXT_BUFFER (text_buffer), &end, utftext, -1);
               g_free (utftext);
             }
           else
-            gtk_text_buffer_insert (GTK_TEXT_BUFFER (text_buffer), &end, string->str, string->len);
+            {
+              if (options.text_data.formatted && !options.common_data.editable)
+                gtk_text_buffer_insert_markup (GTK_TEXT_BUFFER (text_buffer), &end, string->str, string->len);
+              else
+                gtk_text_buffer_insert (GTK_TEXT_BUFFER (text_buffer), &end, string->str, string->len);
+            }
 
           if (options.common_data.tail)
             {
@@ -371,61 +354,46 @@ handle_stdin (GIOChannel * channel, GIOCondition condition, gpointer data)
 static void
 fill_buffer_from_file ()
 {
-  GtkTextIter iter, end;
+  GtkTextIter iter;
 #ifdef HAVE_SOURCEVIEW
   GtkSourceLanguage *lang;
 #endif
-  FILE *f;
-  gchar buf[2048];
-  gint remaining = 0;
+  gchar *buf;
+  gsize len;
+  GError *err = NULL;
 
   if (options.common_data.uri == NULL)
     return;
 
-  f = fopen (options.common_data.uri, "r");
-
-  if (f == NULL)
+  if (!g_file_get_contents (options.common_data.uri, &buf, &len, &err))
     {
-      g_printerr (_("Cannot open file '%s': %s\n"), options.common_data.uri, g_strerror (errno));
+      g_printerr (_("Cannot open file '%s': %s\n"), options.common_data.uri, err->message);
       return;
     }
 
+  if (len <= 0)
+    return;
+
   gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (text_buffer), &iter, 0);
 
-  while (!feof (f))
+  if (!g_utf8_validate (buf, -1, NULL))
     {
-      gint count;
-      const char *leftover;
-      int to_read = 2047 - remaining;
-
-      count = fread (buf + remaining, 1, to_read, f);
-      buf[count + remaining] = '\0';
-
-      g_utf8_validate (buf, count + remaining, &leftover);
-
-      g_assert (g_utf8_validate (buf, leftover - buf, NULL));
-      gtk_text_buffer_insert (GTK_TEXT_BUFFER (text_buffer), &iter, buf, leftover - buf);
-
-      remaining = (buf + remaining + count) - leftover;
-      memmove (buf, leftover, remaining);
-
-      if (remaining > 6 || count < to_read)
-        break;
+      gchar *utftext =
+        g_convert_with_fallback (buf, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL, NULL);
+      if (options.text_data.formatted && !options.common_data.editable)
+        gtk_text_buffer_insert_markup (GTK_TEXT_BUFFER (text_buffer), &iter, utftext, -1);
+      else
+        gtk_text_buffer_insert (GTK_TEXT_BUFFER (text_buffer), &iter, utftext, -1);
+      g_free (utftext);
     }
-
-  if (remaining)
+  else
     {
-      g_printerr (_("Invalid UTF-8 data encountered reading file %s\n"), options.common_data.uri);
-      return;
+      if (options.text_data.formatted && !options.common_data.editable)
+        gtk_text_buffer_insert_markup (GTK_TEXT_BUFFER (text_buffer), &iter, buf, -1);
+      else
+        gtk_text_buffer_insert (GTK_TEXT_BUFFER (text_buffer), &iter, buf, -1);
     }
-
-  /* We had a newline in the buffer to begin with. (The buffer always contains
-   * a newline, so we delete to the end of the buffer to clean up.
-   */
-
-  gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (text_buffer), &end);
-  gtk_text_buffer_delete (GTK_TEXT_BUFFER (text_buffer), &iter, &end);
-  gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (text_buffer), FALSE);
+  g_free (buf);
 
 #ifdef HAVE_SOURCEVIEW
   if (options.source_data.lang)
@@ -450,8 +418,7 @@ fill_buffer_from_stdin ()
 GtkWidget *
 text_create_widget (GtkWidget * dlg)
 {
-  GtkWidget *w;
-  PangoFontDescription *fd;
+  GtkWidget *w, *tv;
 
   w = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (w), GTK_SHADOW_ETCHED_IN);
@@ -459,10 +426,10 @@ text_create_widget (GtkWidget * dlg)
 
 #ifdef HAVE_SOURCEVIEW
   text_buffer = (GObject *) gtk_source_buffer_new (NULL);
-  text_view = gtk_source_view_new_with_buffer (GTK_SOURCE_BUFFER (text_buffer));
+  tv = text_view = gtk_source_view_new_with_buffer (GTK_SOURCE_BUFFER (text_buffer));
 #else
   text_buffer = (GObject *) gtk_text_buffer_new (NULL);
-  text_view = gtk_text_view_new_with_buffer (GTK_TEXT_BUFFER (text_buffer));
+  tv = text_view = gtk_text_view_new_with_buffer (GTK_TEXT_BUFFER (text_buffer));
 #endif
   gtk_widget_set_name (text_view, "yad-text-widget");
   gtk_text_view_set_justification (GTK_TEXT_VIEW (text_view), options.text_data.justify);
@@ -475,30 +442,31 @@ text_create_widget (GtkWidget * dlg)
   if (options.text_data.wrap)
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD_CHAR);
 
-  if (options.text_data.fore)
+  if (options.common_data.font || options.text_data.fore || options.text_data.back)
     {
-#if GTK_CHECK_VERSION(3,0,0)
-      GdkRGBA clr;
-      if (gdk_rgba_parse (&clr, options.text_data.fore))
-        gtk_widget_override_color (text_view, GTK_STATE_FLAG_NORMAL, &clr);
-#else
-      GdkColor clr;
-      if (gdk_color_parse (options.text_data.fore, &clr))
-        gtk_widget_modify_text (text_view, GTK_STATE_NORMAL, &clr);
-#endif
-    }
+      GtkCssProvider *provider;
+      GtkStyleContext *context;
+      GString *css;
 
-  if (options.text_data.back)
-    {
-#if GTK_CHECK_VERSION(3,0,0)
-      GdkRGBA clr;
-      if (gdk_rgba_parse (&clr, options.text_data.back))
-        gtk_widget_override_background_color (text_view, GTK_STATE_FLAG_NORMAL, &clr);
-#else
-      GdkColor clr;
-      if (gdk_color_parse (options.text_data.back, &clr))
-        gtk_widget_modify_base (text_view, GTK_STATE_NORMAL, &clr);
-#endif
+      css = g_string_new (".view, .view text {\n");
+      if (options.common_data.font)
+        {
+          gchar *font = pango_to_css (options.common_data.font);
+          g_string_append_printf (css, "font: %s;\n", font);
+          g_free (font);
+        }
+      if (options.text_data.fore)
+        g_string_append_printf (css, "color: %s;\n", options.text_data.fore);
+      if (options.text_data.back)
+        g_string_append_printf (css, "background-color: %s;\n", options.text_data.back);
+      g_string_append (css, "}\n");
+
+      provider = gtk_css_provider_new ();
+      gtk_css_provider_load_from_data (provider, css->str, -1, NULL);
+      context = gtk_widget_get_style_context (text_view);
+      gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (provider), GTK_STYLE_PROVIDER_PRIORITY_USER);
+
+      g_string_free (css, TRUE);
     }
 
 #ifdef HAVE_SOURCEVIEW
@@ -539,25 +507,11 @@ text_create_widget (GtkWidget * dlg)
     }
 #endif
 
-  /* set font */
-  if (options.common_data.font)
-    fd = pango_font_description_from_string (options.common_data.font);
-  else
-    fd = pango_font_description_from_string ("Monospace");
-
-#if GTK_CHECK_VERSION(3,0,0)
-  gtk_widget_override_font (text_view, fd);
-#else
-  gtk_widget_modify_font (text_view, fd);
-#endif
-  pango_font_description_free (fd);
-
 #ifdef HAVE_SPELL
   if (options.common_data.enable_spell)
     {
-      GtkSpellChecker *spell = gtk_spell_checker_new ();
-      gtk_spell_checker_set_language (spell, options.common_data.spell_lang, NULL);
-      gtk_spell_checker_attach (spell, GTK_TEXT_VIEW (text_view));
+      GspellTextView *spell_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (text_view));
+      gspell_text_view_basic_setup (spell_view);;
     }
 #endif
 
@@ -581,14 +535,14 @@ text_create_widget (GtkWidget * dlg)
       g_signal_connect (G_OBJECT (tag), "event", G_CALLBACK (tag_event_cb), NULL);
 
       /* Create cursors */
-      hand = gdk_cursor_new (GDK_HAND2);
-      normal = gdk_cursor_new (GDK_XTERM);
+      hand = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_HAND2);
+      normal = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_XTERM);
       g_signal_connect (G_OBJECT (text_view), "motion-notify-event", G_CALLBACK (motion_cb), NULL);
 
       g_signal_connect_after (G_OBJECT (text_buffer), "changed", G_CALLBACK (linkify_cb), regex);
     }
 
-  gtk_container_add (GTK_CONTAINER (w), text_view);
+  gtk_container_add (GTK_CONTAINER (w), tv);
 
   if (options.common_data.uri)
     fill_buffer_from_file ();
