@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU General Public License
  * along with YAD. If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2008-2017, Victor Ananjevsky <ananasik@gmail.com>
+ * Copyright (C) 2008-2019, Victor Ananjevsky <ananasik@gmail.com>
  */
 
 #ifndef _YAD_H_
@@ -28,39 +28,40 @@
 #include <gdk/gdkx.h>
 
 #include <gtk/gtk.h>
-#include <glib/gi18n.h>
+#include <gtk/gtkx.h>
 #include <gdk/gdkkeysyms.h>
 
-#if GTK_CHECK_VERSION(3,0,0)
-#include <gtk/gtkx.h>
+#include <glib/gi18n.h>
+#include <glib/gprintf.h>
+
+#ifdef HAVE_HTML
+#include <webkit2/webkit2.h>
 #endif
 
 #ifdef HAVE_SPELL
-#include <gtkspell/gtkspell.h>
+#include <gspell/gspell.h>
 #endif
 
 #ifdef HAVE_SOURCEVIEW
-#include <gtksourceview/gtksourceview.h>
-#include <gtksourceview/gtksourcebuffer.h>
-#include <gtksourceview/gtksourcelanguage.h>
-#include <gtksourceview/gtksourcelanguagemanager.h>
-#include <gtksourceview/gtksourcestylescheme.h>
-#include <gtksourceview/gtksourcestyleschememanager.h>
+#include <gtksourceview/gtksource.h>
+#endif
+
+#ifdef STANDALONE
+#include "defaults.h"
 #endif
 
 G_BEGIN_DECLS
 
-#define YAD_SETTINGS_FILE "yad.conf"
-
 #define YAD_RESPONSE_OK         0
 #define YAD_RESPONSE_CANCEL     1
 #define YAD_RESPONSE_TIMEOUT   	70
 #define YAD_RESPONSE_ESC        -4      /* 252 */
 
-#define YAD_URL_REGEX "(http|https|ftp)://[a-zA-Z0-9./_%#&-]+"
+#define YAD_URL_REGEX "(http|https|ftp|file)://[a-zA-Z0-9./_%#&-]+"
 
 typedef enum {
   YAD_MODE_MESSAGE,
+  YAD_MODE_APP,
   YAD_MODE_CALENDAR,
   YAD_MODE_COLOR,
   YAD_MODE_DND,
@@ -73,9 +74,10 @@ typedef enum {
 #endif
   YAD_MODE_ICONS,
   YAD_MODE_LIST,
-  YAD_MODE_MULTI_PROGRESS,
   YAD_MODE_NOTEBOOK,
+#ifdef HAVE_TRAY
   YAD_MODE_NOTIFICATION,
+#endif
   YAD_MODE_PANED,
   YAD_MODE_PICTURE,
   YAD_MODE_PRINT,
@@ -109,11 +111,13 @@ typedef enum {
   YAD_FIELD_DIR_CREATE,
   YAD_FIELD_MDIR,
   YAD_FIELD_FONT,
+  YAD_FIELD_APP,
   YAD_FIELD_COLOR,
   YAD_FIELD_DATE,
   YAD_FIELD_SCALE,
   YAD_FIELD_BUTTON,
   YAD_FIELD_FULL_BUTTON,
+  YAD_FIELD_LINK,
   YAD_FIELD_LABEL,
   YAD_FIELD_TEXT
 } YadFieldType;
@@ -127,6 +131,7 @@ typedef enum {
   YAD_COLUMN_RADIO,
   YAD_COLUMN_BAR,
   YAD_COLUMN_IMAGE,
+  YAD_COLUMN_TIP,
   YAD_COLUMN_HIDDEN,
   YAD_COLUMN_ATTR_FORE,
   YAD_COLUMN_ATTR_BACK,
@@ -163,6 +168,16 @@ typedef enum {
   YAD_COMPLETE_REGEX
 } YadCompletionType;
 
+typedef enum {
+  YAD_BOOL_FMT_UT,
+  YAD_BOOL_FMT_UY,
+  YAD_BOOL_FMT_UO,
+  YAD_BOOL_FMT_LT,
+  YAD_BOOL_FMT_LY,
+  YAD_BOOL_FMT_LO,
+  YAD_BOOL_FMT_1
+} YadBoolFormat;
+
 typedef struct {
   gchar *name;
   gchar *cmd;
@@ -171,6 +186,7 @@ typedef struct {
 
 typedef struct {
   gchar *name;
+  gchar *tip;
   YadFieldType type;
 } YadField;
 
@@ -205,9 +221,8 @@ typedef struct {
   guint timeout;
   gchar *to_indicator;
   gchar *dialog_text;
-  GtkJustification text_align;
+  gdouble text_align;
   gchar *dialog_image;
-  gboolean image_on_top;
   gchar *icon_theme;
   gchar *expander;
   gint borders;
@@ -218,8 +233,12 @@ typedef struct {
   gboolean escape_ok;
   gboolean always_print;
   gboolean selectable_labels;
+  gboolean keep_icon_size;
   GtkButtonBoxStyle buttons_layout;
   gint def_resp;
+  gboolean use_interp;
+  gchar *interp;
+  gchar *uri_handler;
   /* window settings */
   gboolean sticky;
   gboolean fixed;
@@ -235,6 +254,13 @@ typedef struct {
   gboolean close_on_unfocus;
 } YadData;
 
+typedef struct {
+  gboolean show_fallback;
+  gboolean show_other;
+  gboolean show_all;
+  gboolean extended;
+} YadAppData;
+
 typedef struct {
   gint day;
   gint month;
@@ -249,7 +275,6 @@ typedef struct {
   gboolean use_palette;
   gboolean expand_palette;
   gchar *palette;
-  gboolean extra;
   gboolean alpha;
   YadColorMode mode;
 } YadColorData;
@@ -262,7 +287,6 @@ typedef struct {
 typedef struct {
   gchar *entry_text;
   gchar *entry_label;
-  gboolean hide_text;
   gboolean completion;
   gboolean numeric;
   gchar *licon;
@@ -302,7 +326,6 @@ typedef struct {
   gboolean print_uri;
   gchar *mime;
   gchar *encoding;
-  gchar *uri_cmd;
   gchar *user_agent;
   gchar *user_style;
 } YadHtmlData;
@@ -317,18 +340,16 @@ typedef struct {
   gboolean single_click;
   guint width;
   gchar *term;
-#ifdef HAVE_GIO
   gboolean monitor;
-#endif
 } YadIconsData;
 
 typedef struct {
   GSList *columns;
-  gboolean no_headers;
+  gboolean tree_mode;
   gboolean checkbox;
   gboolean radiobox;
+  gboolean no_headers;
   gboolean print_all;
-  gboolean rules_hint;
   GtkTreeViewGridLines grid_lines;
   gint print_column;
   gint hide_column;
@@ -345,31 +366,31 @@ typedef struct {
   gchar *ellipsize_cols;
   gchar *dclick_action;
   gchar *select_action;
-  gchar *add_action;
+  gchar *row_action;
+  gboolean tree_expanded;
   gboolean regex_search;
   gboolean clickable;
   gboolean no_selection;
   gboolean add_on_top;
+  gboolean simple_tips;
+  gboolean header_tips;
 } YadListData;
 
-typedef struct {
-  GSList *bars;
-  gint watch_bar;
-} YadMultiProgressData;
-
 typedef struct {
   GSList *tabs;
   guint borders;
   GtkPositionType pos;
   guint active;
+  gboolean expand;
 } YadNotebookData;
 
+#ifdef HAVE_TRAY
 typedef struct {
   gboolean middle;
   gboolean hidden;
   gchar *menu;
-  guint icon_size;
 } YadNotificationData;
+#endif
 
 typedef struct {
   GtkOrientation orient;
@@ -387,13 +408,14 @@ typedef struct {
 } YadPrintData;
 
 typedef struct {
+  GSList *bars;
+  gint watch_bar;
   gchar *progress_text;
   gboolean pulsate;
   gboolean autoclose;
 #ifndef G_OS_WIN32
   gboolean autokill;
 #endif
-  guint percentage;
   gboolean rtl;
   gchar *log;
   gboolean log_expanded;
@@ -413,17 +435,19 @@ typedef struct {
   gboolean invert;
   gboolean buttons;
   GSList *marks;
+  gboolean enforce_step;
 } YadScaleData;
 
 typedef struct {
-  gchar *fore;
-  gchar *back;
   gboolean wrap;
   GtkJustification justify;
   gint margins;
   gboolean uri;
   gboolean hide_cursor;
   gchar *uri_color;
+  gboolean formatted;
+  gchar *fore;
+  gchar *back;
 } YadTextData;
 
 #ifdef HAVE_SOURCEVIEW
@@ -448,12 +472,16 @@ typedef struct {
   gdouble align;
   gboolean listen;
   gboolean preview;
+  gboolean large_preview;
   gboolean show_hidden;
   gboolean quoted_output;
   gboolean num_output;
+  gboolean hide_text;
+  gint icon_size;
 #if GLIB_CHECK_VERSION(2,30,0)
   GFormatSizeFlags size_fmt;
 #endif
+  YadBoolFormat bool_fmt;
   YadCompletionType complete;
   GList *filters;
   key_t key;
@@ -469,6 +497,7 @@ typedef struct {
   YadData data;
   YadCommonData common_data;
 
+  YadAppData app_data;
   YadCalendarData calendar_data;
   YadColorData color_data;
   YadDNDData dnd_data;
@@ -481,9 +510,10 @@ typedef struct {
 #endif
   YadIconsData icons_data;
   YadListData list_data;
-  YadMultiProgressData multi_progress_data;
   YadNotebookData notebook_data;
+#ifdef HAVE_TRAY
   YadNotificationData notification_data;
+#endif
   YadPanedData paned_data;
   YadPictureData picture_data;
   YadPrintData print_data;
@@ -505,6 +535,8 @@ typedef struct {
   key_t plug;
   guint tabnum;
 
+  gboolean debug;
+
 #ifndef G_OS_WIN32
   guint kill_parent;
   gboolean print_xid;
@@ -513,29 +545,16 @@ typedef struct {
 } YadOptions;
 
 extern YadOptions options;
+extern GtkIconTheme *yad_icon_theme;
 
-typedef struct {
-  guint width;
-  guint height;
-  guint timeout;
-  gchar *to_indicator;
-  gboolean show_remain;
-  gboolean combo_always_editable;
-  gboolean ignore_unknown;
-  GtkIconTheme *icon_theme;
-  GdkPixbuf *big_fallback_image;
-  GdkPixbuf *small_fallback_image;
-  gchar *term;
-  gchar *open_cmd;
-  gchar *date_format;
-  guint max_tab;
+#ifndef STANDALONE
+extern GSettings *settings;
+#endif
 
-  GtkPrintSettings *print_settings;
-  GtkPageSetup *page_setup;
-} YadSettings;
-
-extern YadSettings settings;
+extern GdkPixbuf *big_fallback_image;
+extern GdkPixbuf *small_fallback_image;
 
+/* TABS */
 typedef struct {
   pid_t pid;
   Window xid;
@@ -544,16 +563,28 @@ typedef struct {
 /* pointer to shared memory for tabbed dialog */
 /* 0 item used for special info: */
 /*   pid - memory id */
-/*   xid - count of registered tabs (for sync) */
+/*   xid - allow plugs to write shmem (for sync) */
 extern YadNTabs *tabs;
-extern gint t_sem;
 
+/* STOCK ITEMS */
+#define YAD_STOCK_COUNT 19
+
+typedef struct {
+  gchar *key;
+  gchar *label;
+  gchar *icon;
+} YadStock;
+
+extern const YadStock yad_stock_items[];
+
+/* FUNCTION PROTOTYPES */
 void yad_options_init (void);
 GOptionContext *yad_create_context (void);
 void yad_set_mode (void);
 void yad_print_result (void);
 void yad_exit (gint id);
 
+GtkWidget *app_create_widget (GtkWidget *dlg);
 GtkWidget *calendar_create_widget (GtkWidget *dlg);
 GtkWidget *color_create_widget (GtkWidget *dlg);
 GtkWidget *entry_create_widget (GtkWidget *dlg);
@@ -565,7 +596,6 @@ GtkWidget *html_create_widget (GtkWidget *dlg);
 #endif
 GtkWidget *icons_create_widget (GtkWidget *dlg);
 GtkWidget *list_create_widget (GtkWidget *dlg);
-GtkWidget *multi_progress_create_widget (GtkWidget *dlg);
 GtkWidget *notebook_create_widget (GtkWidget *dlg);
 GtkWidget *paned_create_widget (GtkWidget *dlg);
 GtkWidget *picture_create_widget (GtkWidget *dlg);
@@ -578,6 +608,7 @@ void notebook_swallow_childs (void);
 void paned_swallow_childs (void);
 void picture_fit_to_window (void);
 
+void app_print_result (void);
 void calendar_print_result (void);
 void color_print_result (void);
 void entry_print_result (void);
@@ -592,7 +623,9 @@ void text_print_result (void);
 
 void dnd_init (GtkWidget *w);
 
+#ifdef HAVE_TRAY
 gint yad_notification_run (void);
+#endif
 gint yad_print_run (void);
 gint yad_about (void);
 
@@ -605,24 +638,42 @@ void read_settings (void);
 void write_settings (void);
 
 void update_preview (GtkFileChooser *chooser, GtkWidget *p);
-void filechooser_mapped (GtkWidget *w, gpointer data);
 
-GdkPixbuf *get_pixbuf (gchar *name, YadIconSize size);
-gchar *get_color (GdkColor *c, guint64 alpha);
+GdkPixbuf *get_pixbuf (gchar *name, YadIconSize size, gboolean force);
+gchar *get_color (GdkRGBA *c);
 
 gchar **split_arg (const gchar *str);
 
 YadNTabs *get_tabs (key_t key, gboolean create);
 
-GtkWidget *get_label (gchar *str, guint border);
+gboolean stock_lookup (gchar *key, YadStock *it);
+
+GtkWidget *get_label (gchar *str, guint border, GtkWidget *w);
 
 gchar *escape_str (gchar *str);
 gchar *escape_char (gchar *str, gchar ch);
 
 gboolean check_complete (GtkEntryCompletion *c, const gchar *key, GtkTreeIter *iter, gpointer data);
 
+void parse_geometry ();
+
+gboolean get_bool_val (gchar *str);
+gchar *print_bool_val (gboolean val);
+
+gint run_command_sync (gchar *cmd, gchar **out);
+void run_command_async (gchar *cmd);
+
+gchar * pango_to_css (gchar *font);
+
+void open_uri (const gchar *uri);
+
+#ifdef HAVE_SPELL
 void show_langs ();
+#endif
+
+#ifdef HAVE_SOURCEVIEW
 void show_themes ();
+#endif
 
 static inline void
 strip_new_line (gchar * str)
